@@ -138,11 +138,8 @@ extern const NSString *kTopLevelUIElementAttribute;   // NSAccessibilityTopLevel
   if ([[self role] isEqualToString:NSAccessibilityStaticTextRole])
     return YES;
     
-  if (mGeckoEditableTextAccessible) {
-    PRUint32 state = 0;
-    mGeckoAccessible->GetState(&state, nsnull);
-    return (state & nsIAccessibleStates::STATE_READONLY) == 0;
-  }
+  if (mGeckoEditableTextAccessible)
+    return (mGeckoAccessible->State() & states::READONLY) == 0;
 
   return NO;
 
