@@ -69,22 +69,12 @@ config/autoconf.mk
 config/mkdepend/Makefile
 config/nspr/Makefile
 config/doxygen.cfg
+config/expandlibs_config.py
 config/tests/src-simple/Makefile
 probes/Makefile
 extensions/Makefile
 "
 
-if [ "$WINCE" ]; then
-  add_makefiles "
-    build/wince/tools/Makefile
-    build/wince/shunt/Makefile
-    build/wince/shunt/include/windows.h
-    build/wince/shunt/include/ymath.h
-    build/wince/shunt/include/stdlib.h
-    build/wince/shunt/include/sys/Makefile
-  "
-fi
-
 if [ "$MOZ_MEMORY" -a "$LIBXUL_SDK" = "" ]; then
   add_makefiles "
     memory/jemalloc/Makefile
