@@ -71,7 +71,6 @@
 #include "nsCSSProperty.h"
 #include "mozilla/css/Declaration.h"
 #include "mozilla/css/StyleRule.h"
-#include "nsUnicharInputStream.h"
 #include "nsCSSStyleSheet.h"
 #include "nsICSSRuleList.h"
 #include "nsIDOMCSSRule.h"
@@ -1153,67 +1152,61 @@ nsHTMLParanoidFragmentSink::CloseContainer(const nsHTMLTag aTag)
       // styleText will hold the text inside the style element.
       nsAutoString styleText;
       nsContentUtils::GetNodeTextContent(style, PR_FALSE, styleText);
-      // Create a unichar input stream for the CSS parser.
-      nsCOMPtr<nsIUnicharInputStream> uin;
-      rv = nsSimpleUnicharStreamFactory::GetInstance()->
-        CreateInstanceFromString(styleText, getter_AddRefs(uin));
+      // Create a sheet to hold the parsed CSS
+      nsRefPtr<nsCSSStyleSheet> sheet;
+      rv = NS_NewCSSStyleSheet(getter_AddRefs(sheet));
       if (NS_SUCCEEDED(rv)) {
-        // Create a sheet to hold the parsed CSS
-        nsRefPtr<nsCSSStyleSheet> sheet;
-        rv = NS_NewCSSStyleSheet(getter_AddRefs(sheet));
+        nsCOMPtr<nsIURI> baseURI = style->GetBaseURI();
+        sheet->SetURIs(mTargetDocument->GetDocumentURI(), nsnull, baseURI);
+        sheet->SetPrincipal(mTargetDocument->NodePrincipal());
+        // Create the CSS parser, and parse the CSS text.
+        nsCSSParser parser(nsnull, sheet);
+        rv = parser.ParseSheet(styleText, mTargetDocument->GetDocumentURI(),
+                               baseURI, mTargetDocument->NodePrincipal(),
+                               0, PR_FALSE);
+        // Mark the sheet as complete.
         if (NS_SUCCEEDED(rv)) {
-          nsCOMPtr<nsIURI> baseURI = style->GetBaseURI();
-          sheet->SetURIs(mTargetDocument->GetDocumentURI(), nsnull, baseURI);
-          sheet->SetPrincipal(mTargetDocument->NodePrincipal());
-          // Create the CSS parser, and parse the CSS text.
-          nsCSSParser parser(nsnull, sheet);
-          rv = parser.Parse(uin, mTargetDocument->GetDocumentURI(),
-                            baseURI, mTargetDocument->NodePrincipal(),
-                            0, PR_FALSE);
-          // Mark the sheet as complete.
-          if (NS_SUCCEEDED(rv)) {
-            NS_ABORT_IF_FALSE(!sheet->IsModified(),
-                              "should not get marked modified during parsing");
-            sheet->SetComplete();
-          }
-          if (NS_SUCCEEDED(rv)) {
-            // Loop through all the rules found in the CSS text
-            PRInt32 ruleCount = sheet->StyleRuleCount();
-            for (PRInt32 i = 0; i < ruleCount; ++i) {
-              nsRefPtr<nsICSSRule> rule;
-              rv = sheet->GetStyleRuleAt(i, *getter_AddRefs(rule));
-              if (NS_FAILED(rv))
-                continue;
-              NS_ASSERTION(rule, "We should have a rule by now");
-              switch (rule->GetType()) {
-                default:
-                  didSanitize = PR_TRUE;
-                  // Ignore these rule types.
-                  break;
-                case nsICSSRule::NAMESPACE_RULE:
-                case nsICSSRule::FONT_FACE_RULE: {
-                  // Append @namespace and @font-face rules verbatim.
-                  nsAutoString cssText;
-                  nsCOMPtr<nsIDOMCSSRule> styleRule = do_QueryInterface(rule);
-                  if (styleRule) {
-                    rv = styleRule->GetCssText(cssText);
-                    if (NS_SUCCEEDED(rv)) {
-                      sanitizedStyleText.Append(cssText);
-                    }
-                  }
-                  break;
-                }
-                case nsICSSRule::STYLE_RULE: {
-                  // For style rules, we will just look for and remove the
-                  // -moz-binding properties.
-                  nsRefPtr<css::StyleRule> styleRule = do_QueryObject(rule);
-                  NS_ASSERTION(styleRule, "Must be a style rule");
-                  nsAutoString decl;
-                  didSanitize = SanitizeStyleRule(styleRule, decl) || didSanitize;
-                  styleRule->GetCssText(decl);
-                  sanitizedStyleText.Append(decl);
+          NS_ABORT_IF_FALSE(!sheet->IsModified(),
+                            "should not get marked modified during parsing");
+          sheet->SetComplete();
+        }
+        if (NS_SUCCEEDED(rv)) {
+          // Loop through all the rules found in the CSS text
+          PRInt32 ruleCount = sheet->StyleRuleCount();
+          for (PRInt32 i = 0; i < ruleCount; ++i) {
+            nsRefPtr<css::Rule> rule;
+            rv = sheet->GetStyleRuleAt(i, *getter_AddRefs(rule));
+            if (NS_FAILED(rv))
+              continue;
+            NS_ASSERTION(rule, "We should have a rule by now");
+            switch (rule->GetType()) {
+            default:
+              didSanitize = PR_TRUE;
+              // Ignore these rule types.
+              break;
+            case css::Rule::NAMESPACE_RULE:
+            case css::Rule::FONT_FACE_RULE: {
+              // Append @namespace and @font-face rules verbatim.
+              nsAutoString cssText;
+              nsCOMPtr<nsIDOMCSSRule> styleRule = do_QueryInterface(rule);
+              if (styleRule) {
+                rv = styleRule->GetCssText(cssText);
+                if (NS_SUCCEEDED(rv)) {
+                  sanitizedStyleText.Append(cssText);
                 }
               }
+              break;
+            }
+            case css::Rule::STYLE_RULE: {
+              // For style rules, we will just look for and remove the
+              // -moz-binding properties.
+              nsRefPtr<css::StyleRule> styleRule = do_QueryObject(rule);
+              NS_ASSERTION(styleRule, "Must be a style rule");
+              nsAutoString decl;
+              didSanitize = SanitizeStyleRule(styleRule, decl) || didSanitize;
+              styleRule->GetCssText(decl);
+              sanitizedStyleText.Append(decl);
+            }
             }
           }
         }
