@@ -49,7 +49,6 @@
 #include "nsIXMLHttpRequest.h"
 
 // Other includes
-#include "nsAutoLock.h"
 #include "nsComponentManagerUtils.h"
 #include "nsIClassInfoImpl.h"
 #include "nsThreadUtils.h"
@@ -66,6 +65,8 @@
 #include "nsDOMWorkerXHR.h"
 #include "nsDOMWorkerXHRProxiedFunctions.h"
 
+using namespace mozilla;
+
 #define MAX_XHR_LISTENER_TYPE nsDOMWorkerXHREventTarget::sMaxXHREventTypes
 #define MAX_UPLOAD_LISTENER_TYPE nsDOMWorkerXHREventTarget::sMaxUploadEventTypes
 
@@ -181,7 +182,7 @@ public:
     nsRefPtr<nsDOMWorkerXHREvent> lastProgressOrLoadEvent;
 
     if (!mProxy->mCanceled) {
-      nsAutoLock lock(mProxy->mWorkerXHR->Lock());
+      MutexAutoLock lock(mProxy->mWorkerXHR->GetLock());
       mProxy->mLastProgressOrLoadEvent.swap(lastProgressOrLoadEvent);
       if (mProxy->mCanceled) {
         return NS_ERROR_ABORT;
@@ -375,7 +376,7 @@ nsDOMWorkerXHRProxy::Destroy()
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
 
   {
-    nsAutoLock lock(mWorkerXHR->Lock());
+    MutexAutoLock lock(mWorkerXHR->GetLock());
 
     mCanceled = PR_TRUE;
 
@@ -465,7 +466,7 @@ nsDOMWorkerXHRProxy::DestroyInternal()
     // necko has fired its OnStartRequest notification. Guard against that here.
     nsRefPtr<nsDOMWorkerXHRFinishSyncXHRRunnable> syncFinishedRunnable;
     {
-      nsAutoLock lock(mWorkerXHR->Lock());
+      MutexAutoLock lock(mWorkerXHR->GetLock());
       mSyncFinishedRunnable.swap(syncFinishedRunnable);
     }
 
@@ -580,7 +581,7 @@ nsDOMWorkerXHRProxy::HandleWorkerEvent(nsDOMWorkerXHREvent* aEvent,
   NS_ASSERTION(aEvent, "Should not be null!");
 
   {
-    nsAutoLock lock(mWorkerXHR->Lock());
+    MutexAutoLock lock(mWorkerXHR->GetLock());
 
     if (mCanceled ||
         (aEvent->mChannelID != -1 && aEvent->mChannelID != mChannelID)) {
@@ -605,7 +606,7 @@ nsDOMWorkerXHRProxy::HandleWorkerEvent(nsDOMWorkerXHREvent* aEvent,
     progressInfo = nsnull;
 
     // Dummy memory barrier.
-    nsAutoLock lock(mWorkerXHR->Lock());
+    MutexAutoLock lock(mWorkerXHR->GetLock());
   }
 
   nsIDOMEventTarget* target = aUploadEvent ?
@@ -683,7 +684,7 @@ nsDOMWorkerXHRProxy::DispatchPrematureAbortEvents(PRUint32 aType,
 }
 
 nsresult
-nsDOMWorkerXHRProxy::MaybeDispatchPrematureAbortEvents(PRBool aFromOpenRequest)
+nsDOMWorkerXHRProxy::MaybeDispatchPrematureAbortEvents(PRBool aFromOpen)
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
 
@@ -698,7 +699,7 @@ nsDOMWorkerXHRProxy::MaybeDispatchPrematureAbortEvents(PRBool aFromOpenRequest)
                                       nsnull);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    if (aFromOpenRequest) {
+    if (aFromOpen) {
       rv = DispatchPrematureAbortEvents(LISTENER_TYPE_ABORT, target,
                                         mDownloadProgressInfo);
       NS_ENSURE_SUCCESS(rv, rv);
@@ -769,7 +770,7 @@ nsDOMWorkerXHRProxy::HandleEvent(nsIDOMEvent* aEvent)
 
     NS_ASSERTION(!syncFinishedRunnable, "This shouldn't be set!");
 
-    nsAutoLock lock(mWorkerXHR->Lock());
+    MutexAutoLock lock(mWorkerXHR->GetLock());
     mSyncFinishedRunnable.swap(syncFinishedRunnable);
   }
   else {
@@ -811,7 +812,7 @@ nsDOMWorkerXHRProxy::HandleEvent(nsIDOMEvent* aEvent)
     NS_ENSURE_TRUE(runnable, NS_ERROR_OUT_OF_MEMORY);
 
     {
-      nsAutoLock lock(mWorkerXHR->Lock());
+      MutexAutoLock lock(mWorkerXHR->GetLock());
 
       if (mCanceled) {
         return NS_ERROR_ABORT;
@@ -866,17 +867,17 @@ nsDOMWorkerXHRProxy::HandleEventRunnable(nsIRunnable* aRunnable)
 }
 
 nsresult
-nsDOMWorkerXHRProxy::OpenRequest(const nsACString& aMethod,
-                                 const nsACString& aUrl,
-                                 PRBool aAsync,
-                                 const nsAString& aUser,
-                                 const nsAString& aPassword)
+nsDOMWorkerXHRProxy::Open(const nsACString& aMethod,
+                          const nsACString& aUrl,
+                          PRBool aAsync,
+                          const nsAString& aUser,
+                          const nsAString& aPassword)
 {
   if (!NS_IsMainThread()) {
     mSyncRequest = !aAsync;
 
     // Always do async behind the scenes!
-    RUN_PROXIED_FUNCTION(OpenRequest,
+    RUN_PROXIED_FUNCTION(Open,
                          (aMethod, aUrl, PR_TRUE, aUser, aPassword));
     return NS_OK;
   }
@@ -888,12 +889,12 @@ nsDOMWorkerXHRProxy::OpenRequest(const nsACString& aMethod,
   nsresult rv = MaybeDispatchPrematureAbortEvents(PR_TRUE);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = mXHR->OpenRequest(aMethod, aUrl, aAsync, aUser, aPassword);
+  rv = mXHR->Open(aMethod, aUrl, aAsync, aUser, aPassword);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  // Do this after OpenRequest is called so that we will continue to run events
-  // from the old channel if OpenRequest fails. Any events generated by the
-  // OpenRequest method will always run regardless of channel ID.
+  // Do this after Open is called so that we will continue to run events
+  // from the old channel if Open fails. Any events generated by the
+  // Open method will always run regardless of channel ID.
   mChannelID++;
 
   return NS_OK;
@@ -957,7 +958,7 @@ nsDOMWorkerXHRProxy::Send(nsIVariant* aBody)
     mSyncXHRThread = NS_GetCurrentThread();
     NS_ENSURE_TRUE(mSyncXHRThread, NS_ERROR_FAILURE);
 
-    nsAutoLock lock(mWorkerXHR->Lock());
+    MutexAutoLock lock(mWorkerXHR->GetLock());
 
     if (mCanceled) {
       return NS_ERROR_ABORT;
@@ -982,7 +983,7 @@ nsDOMWorkerXHRProxy::SendAsBinary(const nsAString& aBody)
     mSyncXHRThread = NS_GetCurrentThread();
     NS_ENSURE_TRUE(mSyncXHRThread, NS_ERROR_FAILURE);
 
-    nsAutoLock lock(mWorkerXHR->Lock());
+    MutexAutoLock lock(mWorkerXHR->GetLock());
 
     if (mCanceled) {
       return NS_ERROR_ABORT;
@@ -1089,7 +1090,7 @@ nsDOMWorkerXHRProxy::GetStatus(nsresult* _retval)
 }
 
 nsresult
-nsDOMWorkerXHRProxy::GetReadyState(PRInt32* _retval)
+nsDOMWorkerXHRProxy::GetReadyState(PRUint16* _retval)
 {
   NS_ASSERTION(_retval, "Null pointer!");
 
