@@ -132,6 +132,40 @@ class GeckoSurfaceView
      * Called on main thread
      */
 
+    public void draw(SurfaceHolder holder, ByteBuffer buffer) {
+        if (buffer == null || buffer.capacity() != (mWidth * mHeight * 2))
+            return;
+
+        synchronized (mSoftwareBuffer) {
+            if (buffer != mSoftwareBuffer || mSoftwareBufferCopy == null)
+                return;
+
+            Canvas c = holder.lockCanvas();
+            if (c == null)
+                return;
+            mSoftwareBufferCopy.copyPixelsFromBuffer(buffer);
+            c.drawBitmap(mSoftwareBufferCopy, 0, 0, null);
+            holder.unlockCanvasAndPost(c);
+        }
+    }
+
+    public void draw(SurfaceHolder holder, Bitmap bitmap) {
+        if (bitmap == null ||
+            bitmap.getWidth() != mWidth || bitmap.getHeight() != mHeight)
+            return;
+
+        synchronized (mSoftwareBitmap) {
+            if (bitmap != mSoftwareBitmap)
+                return;
+
+            Canvas c = holder.lockCanvas();
+            if (c == null)
+                return;
+            c.drawBitmap(bitmap, 0, 0, null);
+            holder.unlockCanvasAndPost(c);
+        }
+    }
+
     public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
         if (mShowingSplashScreen)
             drawSplashScreen(holder, width, height);
@@ -142,14 +176,20 @@ class GeckoSurfaceView
                 Log.w("GeckoAppJava", "surfaceChanged while mInDrawing is true!");
             }
 
-            if (width == 0 || height == 0)
+            boolean invalidSize;
+
+            if (width == 0 || height == 0) {
+                mSoftwareBitmap = null;
                 mSoftwareBuffer = null;
-            else if (mSoftwareBuffer == null ||
-                     mSoftwareBuffer.capacity() < (width * height * 2) ||
-                     mWidth != width || mHeight != height)
-                mSoftwareBuffer = ByteBuffer.allocateDirect(width * height * 2);
-            boolean doSyncDraw = mDrawMode == DRAW_2D &&
-                mSoftwareBuffer != null &&
+                mSoftwareBufferCopy = null;
+                invalidSize = true;
+            } else {
+                invalidSize = false;
+            }
+
+            boolean doSyncDraw =
+                mDrawMode == DRAW_2D &&
+                !invalidSize &&
                 GeckoApp.checkLaunchState(GeckoApp.LaunchState.GeckoRunning);
             mSyncDraw = doSyncDraw;
 
@@ -167,9 +207,6 @@ class GeckoSurfaceView
                                           metrics.widthPixels, metrics.heightPixels);
             GeckoAppShell.sendEventToGecko(e);
 
-            if (mSoftwareBuffer != null)
-                GeckoAppShell.scheduleRedraw();
-
             if (!doSyncDraw) {
                 if (mDrawMode == DRAW_GLES_2 || mShowingSplashScreen)
                     return;
@@ -177,23 +214,26 @@ class GeckoSurfaceView
                 c.drawARGB(255, 255, 255, 255);
                 holder.unlockCanvasAndPost(c);
                 return;
+            } else {
+                GeckoAppShell.scheduleRedraw();
             }
         } finally {
             mSurfaceLock.unlock();
         }
 
-        ByteBuffer bb = null;
+        Object syncDrawObject = null;
         try {
-            bb = mSyncBuf.take();
+            syncDrawObject = mSyncDraws.take();
         } catch (InterruptedException ie) {
-            Log.e("GeckoAppJava", "Threw exception while getting sync buf: ", ie);
+            Log.e("GeckoAppJava", "Threw exception while getting sync draw bitmap/buffer: ", ie);
         }
-        if (bb != null && bb.capacity() == (width * height * 2)) {
-            mSoftwareBitmap = Bitmap.createBitmap(mWidth, mHeight, Bitmap.Config.RGB_565);
-            mSoftwareBitmap.copyPixelsFromBuffer(bb);
-            Canvas c = holder.lockCanvas();
-            c.drawBitmap(mSoftwareBitmap, 0, 0, null);
-            holder.unlockCanvasAndPost(c);
+        if (syncDrawObject != null) {
+            if (syncDrawObject instanceof Bitmap)
+                draw(holder, (Bitmap)syncDrawObject);
+            else
+                draw(holder, (ByteBuffer)syncDrawObject);
+        } else {
+            Log.e("GeckoSurfaceViewJava", "Synchronised draw object is null");
         }
     }
 
@@ -209,11 +249,37 @@ class GeckoSurfaceView
         Log.i("GeckoAppJava", "surface destroyed");
         mSurfaceValid = false;
         mSoftwareBuffer = null;
+        mSoftwareBufferCopy = null;
+        mSoftwareBitmap = null;
         GeckoEvent e = new GeckoEvent(GeckoEvent.SURFACE_DESTROYED);
         GeckoAppShell.sendEventToGecko(e);
     }
 
+    public Bitmap getSoftwareDrawBitmap() {
+        if (mSoftwareBitmap == null ||
+            mSoftwareBitmap.getHeight() != mHeight ||
+            mSoftwareBitmap.getWidth() != mWidth) {
+            mSoftwareBitmap = Bitmap.createBitmap(mWidth, mHeight, Bitmap.Config.RGB_565);
+        }
+
+        mDrawMode = DRAW_2D;
+        return mSoftwareBitmap;
+    }
+
     public ByteBuffer getSoftwareDrawBuffer() {
+        // We store pixels in 565 format, so two bytes per pixel (explaining
+        // the * 2 in the following check/allocation)
+        if (mSoftwareBuffer == null ||
+            mSoftwareBuffer.capacity() != (mWidth * mHeight * 2)) {
+            mSoftwareBuffer = ByteBuffer.allocateDirect(mWidth * mHeight * 2);
+        }
+
+        if (mSoftwareBufferCopy == null ||
+            mSoftwareBufferCopy.getHeight() != mHeight ||
+            mSoftwareBufferCopy.getWidth() != mWidth) {
+            mSoftwareBufferCopy = Bitmap.createBitmap(mWidth, mHeight, Bitmap.Config.RGB_565);
+        }
+
         mDrawMode = DRAW_2D;
         return mSoftwareBuffer;
     }
@@ -290,19 +356,19 @@ class GeckoSurfaceView
      * unless you're in SurfaceChanged, in which case the canvas was already
      * locked. Surface lock -> Canvas lock will lead to AB-BA deadlocks.
      */
-    public void draw2D(ByteBuffer buffer, int stride) {
-        // mSurfaceLock ensures that we get mSyncDraw/mSoftwareBuffer/etc.
+    public void draw2D(Bitmap bitmap, int width, int height) {
+        // mSurfaceLock ensures that we get mSyncDraw/mSoftwareBitmap/etc.
         // set correctly before determining whether we should do a sync draw
         mSurfaceLock.lock();
         try {
             if (mSyncDraw) {
-                if (buffer != mSoftwareBuffer || stride != (mWidth * 2))
+                if (bitmap != mSoftwareBitmap || width != mWidth || height != mHeight)
                     return;
                 mSyncDraw = false;
                 try {
-                    mSyncBuf.put(buffer);
+                    mSyncDraws.put(bitmap);
                 } catch (InterruptedException ie) {
-                    Log.e("GeckoAppJava", "Threw exception while getting sync buf: ", ie);
+                    Log.e("GeckoAppJava", "Threw exception while getting sync draws queue: ", ie);
                 }
                 return;
             }
@@ -310,29 +376,28 @@ class GeckoSurfaceView
             mSurfaceLock.unlock();
         }
 
-        if (buffer != mSoftwareBuffer || stride != (mWidth * 2))
-            return;
-        Canvas c = getHolder().lockCanvas();
-        if (c == null)
-            return;
-        if (buffer != mSoftwareBuffer || stride != (mWidth * 2)) {
-            /* We're screwed. Fill it with white and hope it isn't too noticable
-             * This could potentially happen if this function is called
-             * right before mSurfaceLock is locked in SurfaceChanged.
-             * However, I've never actually seen this code get hit.
-             */
-            c.drawARGB(255, 255, 255, 255);
-            getHolder().unlockCanvasAndPost(c);
-            return;
+        draw(getHolder(), bitmap);
+    }
+
+    public void draw2D(ByteBuffer buffer, int stride) {
+        mSurfaceLock.lock();
+        try {
+            if (mSyncDraw) {
+                if (buffer != mSoftwareBuffer || stride != (mWidth * 2))
+                    return;
+                mSyncDraw = false;
+                try {
+                    mSyncDraws.put(buffer);
+                } catch (InterruptedException ie) {
+                    Log.e("GeckoAppJava", "Threw exception while getting sync bitmaps queue: ", ie);
+                }
+                return;
+            }
+        } finally {
+            mSurfaceLock.unlock();
         }
-        if (mSoftwareBitmap == null ||
-            mSoftwareBitmap.getHeight() != mHeight ||
-            mSoftwareBitmap.getWidth() != mWidth) {
-            mSoftwareBitmap = Bitmap.createBitmap(mWidth, mHeight, Bitmap.Config.RGB_565);
-        }
-        mSoftwareBitmap.copyPixelsFromBuffer(mSoftwareBuffer);
-        c.drawBitmap(mSoftwareBitmap, 0, 0, null);
-        getHolder().unlockCanvasAndPost(c);
+
+        draw(getHolder(), buffer);
     }
 
     @Override
@@ -383,7 +448,10 @@ class GeckoSurfaceView
             outAttrs.imeOptions = EditorInfo.IME_ACTION_SEND;
         else if (mIMEActionHint != null && mIMEActionHint.length() != 0)
             outAttrs.actionLabel = mIMEActionHint;
-            
+
+        if (mIMELandscapeFS == false)
+            outAttrs.imeOptions |= EditorInfo.IME_FLAG_NO_EXTRACT_UI;
+
         inputConnection.reset();
         return inputConnection;
     }
@@ -413,10 +481,47 @@ class GeckoSurfaceView
         GeckoAppShell.sendEventToGecko(new GeckoEvent(event));
     }
 
+    private class GeocoderTask extends AsyncTask<Location, Void, Void> {
+        protected Void doInBackground(Location... location) {
+            try {
+                List<Address> addresses = mGeocoder.getFromLocation(location[0].getLatitude(),
+                                                                    location[0].getLongitude(), 1);
+                // grab the first address.  in the future,
+                // may want to expose multiple, or filter
+                // for best.
+                mLastGeoAddress = addresses.get(0);
+                GeckoAppShell.sendEventToGecko(new GeckoEvent(location[0], mLastGeoAddress));
+            } catch (Exception e) {
+                Log.w("GeckoSurfaceView", "GeocoderTask "+e);
+            }
+            return null;
+        }
+    }
+
     // geolocation
     public void onLocationChanged(Location location)
     {
-        GeckoAppShell.sendEventToGecko(new GeckoEvent(location));
+        if (mGeocoder == null)
+            mGeocoder = new Geocoder(getContext(), Locale.getDefault());
+
+        if (mLastGeoAddress == null) {
+            new GeocoderTask().execute(location);
+        }
+        else {
+            float[] results = new float[1];
+            Location.distanceBetween(location.getLatitude(),
+                                     location.getLongitude(),
+                                     mLastGeoAddress.getLatitude(),
+                                     mLastGeoAddress.getLongitude(),
+                                     results);
+            // pfm value.  don't want to slam the
+            // geocoder with very similar values, so
+            // only call after about 100m
+            if (results[0] > 100)
+                new GeocoderTask().execute(location);
+        }
+
+        GeckoAppShell.sendEventToGecko(new GeckoEvent(location, mLastGeoAddress));
     }
 
     public void onProviderDisabled(String provider)
@@ -494,6 +599,7 @@ class GeckoSurfaceView
         // KeyListener returns true if it handled the event for us.
         if (mIMEState == IME_STATE_DISABLED ||
             keyCode == KeyEvent.KEYCODE_ENTER ||
+            (event.getFlags() & KeyEvent.FLAG_SOFT_KEYBOARD) != 0 ||
             !mKeyListener.onKeyDown(this, mEditable, keyCode, event))
             GeckoAppShell.sendEventToGecko(new GeckoEvent(event));
         return true;
@@ -511,6 +617,7 @@ class GeckoSurfaceView
         }
         if (mIMEState == IME_STATE_DISABLED ||
             keyCode == KeyEvent.KEYCODE_ENTER ||
+            (event.getFlags() & KeyEvent.FLAG_SOFT_KEYBOARD) != 0 ||
             !mKeyListener.onKeyUp(this, mEditable, keyCode, event))
             GeckoAppShell.sendEventToGecko(new GeckoEvent(event));
         return true;
@@ -577,20 +684,25 @@ class GeckoSurfaceView
     public static final int IME_STATE_DISABLED = 0;
     public static final int IME_STATE_ENABLED = 1;
     public static final int IME_STATE_PASSWORD = 2;
+    public static final int IME_STATE_PLUGIN = 3;
 
     GeckoInputConnection inputConnection;
     KeyListener mKeyListener;
     Editable mEditable;
     Editable.Factory mEditableFactory;
-    boolean mIMEFocus;
     int mIMEState;
     String mIMETypeHint;
     String mIMEActionHint;
+    boolean mIMELandscapeFS;
 
     // Software rendering
-    ByteBuffer mSoftwareBuffer;
     Bitmap mSoftwareBitmap;
+    ByteBuffer mSoftwareBuffer;
+    Bitmap mSoftwareBufferCopy;
 
-    final SynchronousQueue<ByteBuffer> mSyncBuf = new SynchronousQueue<ByteBuffer>();
+    Geocoder mGeocoder;
+    Address  mLastGeoAddress;
+
+    final SynchronousQueue<Object> mSyncDraws = new SynchronousQueue<Object>();
 }
 
