@@ -40,15 +40,12 @@
 //#define USEWEAKREFS // (haven't quite figured that out yet)
 
 #include "nsWindowWatcher.h"
+#include "nsAutoWindowStateHelper.h"
 
-#include "nsAutoLock.h"
 #include "nsCRT.h"
 #include "nsNetUtil.h"
-#include "nsPrompt.h"
-#include "nsPromptService.h"
 #include "nsWWJSUtils.h"
 #include "plstr.h"
-#include "nsIContentUtils.h"
 
 #include "nsIBaseWindow.h"
 #include "nsIDocShell.h"
@@ -62,11 +59,11 @@
 #include "nsIDOMChromeWindow.h"
 #include "nsIDOMWindowInternal.h"
 #include "nsIDOMModalContentWindow.h"
+#include "nsIPrompt.h"
 #include "nsIScriptObjectPrincipal.h"
 #include "nsIScreen.h"
 #include "nsIScreenManager.h"
 #include "nsIScriptContext.h"
-#include "nsIGenericFactory.h"
 #include "nsIJSContextStack.h"
 #include "nsIObserverService.h"
 #include "nsIScriptGlobalObject.h"
@@ -86,22 +83,23 @@
 #include "nsIWindowProvider.h"
 #include "nsIMutableArray.h"
 #include "nsISupportsArray.h"
-#include "nsIDeviceContext.h"
 #include "nsIDOMStorageObsolete.h"
 #include "nsIDOMStorage.h"
 #include "nsPIDOMStorage.h"
 #include "nsIWidget.h"
 #include "nsFocusManager.h"
+#include "nsIPresShell.h"
+#include "nsPresContext.h"
 
 #include "nsIPrefBranch.h"
 #include "nsIPrefService.h"
 
-#include "jsinterp.h" // for js_AllocStack() and js_FreeStack()
-
 #ifdef USEWEAKREFS
 #include "nsIWeakReference.h"
 #endif
 
+using namespace mozilla;
+
 static const char *sJSStackContractID="@mozilla.org/js/xpc/ContextStack;1";
 
 /****************************************************************
@@ -331,16 +329,15 @@ nsresult JSContextAutoPopper::Push(JSContext *cx)
 
 NS_IMPL_ADDREF(nsWindowWatcher)
 NS_IMPL_RELEASE(nsWindowWatcher)
-NS_IMPL_QUERY_INTERFACE4(nsWindowWatcher,
+NS_IMPL_QUERY_INTERFACE3(nsWindowWatcher,
                          nsIWindowWatcher,
                          nsIPromptFactory,
-                         nsIAuthPromptAdapterFactory,
                          nsPIWindowWatcher)
 
 nsWindowWatcher::nsWindowWatcher() :
         mEnumeratorList(),
         mOldestWindow(0),
-        mListLock(0)
+        mListLock("nsWindowWatcher.mListLock")
 {
 }
 
@@ -349,17 +346,11 @@ nsWindowWatcher::~nsWindowWatcher()
   // delete data
   while (mOldestWindow)
     RemoveWindow(mOldestWindow);
-
-  if (mListLock)
-    PR_DestroyLock(mListLock);
 }
 
 nsresult
 nsWindowWatcher::Init()
 {
-  mListLock = PR_NewLock();
-  if (!mListLock)
-    return NS_ERROR_OUT_OF_MEMORY;
   return NS_OK;
 }
 
@@ -520,9 +511,7 @@ nsWindowWatcher::OpenWindowJSInternal(nsIDOMWindow *aParent,
   NS_ENSURE_ARG_POINTER(_retval);
   *_retval = 0;
 
-  nsCOMPtr<nsIContentUtils> utils =
-    do_GetService("@mozilla.org/content/contentutils;1");
-  if (utils && !utils->IsSafeToRunScript()) {
+  if (!nsContentUtils::IsSafeToRunScript()) {
     return NS_ERROR_FAILURE;
   }
 
@@ -558,7 +547,15 @@ nsWindowWatcher::OpenWindowJSInternal(nsIDOMWindow *aParent,
 
   // no extant window? make a new one.
 
-  nsCOMPtr<nsIDOMChromeWindow> chromeParent(do_QueryInterface(aParent));
+  // If no parent, consider it chrome.
+  PRBool hasChromeParent = PR_TRUE;
+  if (aParent) {
+    // Check if the parent document has chrome privileges.
+    nsCOMPtr<nsIDOMDocument> domdoc;
+    aParent->GetDocument(getter_AddRefs(domdoc));
+    nsCOMPtr<nsIDocument> doc = do_QueryInterface(domdoc);
+    hasChromeParent = doc && nsContentUtils::IsChromeDoc(doc);
+  }
 
   // Make sure we call CalculateChromeFlags() *before* we push the
   // callee context onto the context stack so that
@@ -566,7 +563,7 @@ nsWindowWatcher::OpenWindowJSInternal(nsIDOMWindow *aParent,
   // security checks.
   chromeFlags = CalculateChromeFlags(features.get(), featuresSpecified,
                                      aDialog, uriToLoadIsChrome,
-                                     !aParent || chromeParent);
+                                     hasChromeParent);
 
   // If we're not called through our JS version of the API, and we got
   // our internal modal option, treat the window we're opening as a
@@ -584,6 +581,8 @@ nsWindowWatcher::OpenWindowJSInternal(nsIDOMWindow *aParent,
   nsCOMPtr<nsIScriptSecurityManager>
     sm(do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID));
 
+  NS_ENSURE_TRUE(sm, NS_ERROR_FAILURE);
+
   // Remember who's calling us. This code used to assume a null
   // subject principal if it failed to get the principal, but that's
   // just not safe, so bail on errors here.
@@ -599,7 +598,7 @@ nsWindowWatcher::OpenWindowJSInternal(nsIDOMWindow *aParent,
 
   JSContext *cx = GetJSContextFromWindow(aParent);
 
-  if (isCallerChrome && !chromeParent && cx) {
+  if (isCallerChrome && !hasChromeParent && cx) {
     // open() is called from chrome on a non-chrome window, push
     // the context of the callee onto the context stack to
     // prevent the caller's priveleges from leaking into code
@@ -627,7 +626,7 @@ nsWindowWatcher::OpenWindowJSInternal(nsIDOMWindow *aParent,
         NS_ASSERTION(aParent, "We've _got_ to have a parent here!");
 
         nsCOMPtr<nsIDOMWindow> newWindow;
-        rv = provider->ProvideWindow(aParent, chromeFlags,
+        rv = provider->ProvideWindow(aParent, chromeFlags, aCalledFromJS,
                                      sizeSpec.PositionSpecified(),
                                      sizeSpec.SizeSpecified(),
                                      uriToLoad, name, features, &windowIsNew,
@@ -675,7 +674,7 @@ nsWindowWatcher::OpenWindowJSInternal(nsIDOMWindow *aParent,
     // isn't a chrome window.  Otherwise we can end up in a bizarre situation
     // where we can't shut down because an invisible window is open.  If
     // someone tries to do this, throw.
-    if (!chromeParent && (chromeFlags & nsIWebBrowserChrome::CHROME_MODAL)) {
+    if (!hasChromeParent && (chromeFlags & nsIWebBrowserChrome::CHROME_MODAL)) {
       PRBool parentVisible = PR_TRUE;
       nsCOMPtr<nsIBaseWindow> parentWindow(do_GetInterface(parentTreeOwner));
       nsCOMPtr<nsIWidget> parentWidget;
@@ -846,10 +845,9 @@ nsWindowWatcher::OpenWindowJSInternal(nsIDOMWindow *aParent,
     // Notify observers that the window is open and ready.
     // The window has not yet started to load a document.
     nsCOMPtr<nsIObserverService> obsSvc =
-      do_GetService("@mozilla.org/observer-service;1");
-    if (obsSvc) {
+      mozilla::services::GetObserverService();
+    if (obsSvc)
       obsSvc->NotifyObservers(*_retval, "toplevel-window-ready", nsnull);
-    }
   }
 
   // Now we have to set the right opener principal on the new window.  Note
@@ -962,7 +960,8 @@ nsWindowWatcher::OpenWindowJSInternal(nsIDOMWindow *aParent,
 
   if (subjectPrincipal && parentDocShell) {
     nsCOMPtr<nsIDOMStorage> storage;
-    parentDocShell->GetSessionStorageForPrincipal(subjectPrincipal, PR_FALSE,
+    parentDocShell->GetSessionStorageForPrincipal(subjectPrincipal,
+                                                  EmptyString(), PR_FALSE,
                                                   getter_AddRefs(storage));
     nsCOMPtr<nsPIDOMStorage> piStorage =
       do_QueryInterface(storage);
@@ -1034,17 +1033,18 @@ NS_IMETHODIMP
 nsWindowWatcher::RegisterNotification(nsIObserver *aObserver)
 {
   // just a convenience method; it delegates to nsIObserverService
-  nsresult rv;
 
   if (!aObserver)
     return NS_ERROR_INVALID_ARG;
   
-  nsCOMPtr<nsIObserverService> os(do_GetService("@mozilla.org/observer-service;1", &rv));
-  if (os) {
-    rv = os->AddObserver(aObserver, "domwindowopened", PR_FALSE);
-    if (NS_SUCCEEDED(rv))
-      rv = os->AddObserver(aObserver, "domwindowclosed", PR_FALSE);
-  }
+  nsCOMPtr<nsIObserverService> os = mozilla::services::GetObserverService();
+  if (!os)
+    return NS_ERROR_FAILURE;
+
+  nsresult rv = os->AddObserver(aObserver, "domwindowopened", PR_FALSE);
+  if (NS_SUCCEEDED(rv))
+    rv = os->AddObserver(aObserver, "domwindowclosed", PR_FALSE);
+
   return rv;
 }
 
@@ -1052,17 +1052,18 @@ NS_IMETHODIMP
 nsWindowWatcher::UnregisterNotification(nsIObserver *aObserver)
 {
   // just a convenience method; it delegates to nsIObserverService
-  nsresult rv;
 
   if (!aObserver)
     return NS_ERROR_INVALID_ARG;
   
-  nsCOMPtr<nsIObserverService> os(do_GetService("@mozilla.org/observer-service;1", &rv));
-  if (os) {
-    os->RemoveObserver(aObserver, "domwindowopened");
-    os->RemoveObserver(aObserver, "domwindowclosed");
-  }
-  return rv;
+  nsCOMPtr<nsIObserverService> os = mozilla::services::GetObserverService();
+  if (!os)
+    return NS_ERROR_FAILURE;
+
+  os->RemoveObserver(aObserver, "domwindowopened");
+  os->RemoveObserver(aObserver, "domwindowclosed");
+
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -1071,7 +1072,7 @@ nsWindowWatcher::GetWindowEnumerator(nsISimpleEnumerator** _retval)
   if (!_retval)
     return NS_ERROR_INVALID_ARG;
 
-  nsAutoLock lock(mListLock);
+  MutexAutoLock lock(mListLock);
   nsWatcherWindowEnumerator *enumerator = new nsWatcherWindowEnumerator(this);
   if (enumerator)
     return CallQueryInterface(enumerator, _retval);
@@ -1082,54 +1083,46 @@ nsWindowWatcher::GetWindowEnumerator(nsISimpleEnumerator** _retval)
 NS_IMETHODIMP
 nsWindowWatcher::GetNewPrompter(nsIDOMWindow *aParent, nsIPrompt **_retval)
 {
-  return NS_NewPrompter(_retval, aParent);
+  // This is for backwards compat only. Callers should just use the prompt service directly.
+  nsresult rv;
+  nsCOMPtr<nsIPromptFactory> factory = do_GetService("@mozilla.org/prompter;1", &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+  return factory->GetPrompt(aParent, NS_GET_IID(nsIPrompt), reinterpret_cast<void**>(_retval));
 }
 
 NS_IMETHODIMP
 nsWindowWatcher::GetNewAuthPrompter(nsIDOMWindow *aParent, nsIAuthPrompt **_retval)
 {
-  return NS_NewAuthPrompter(_retval, aParent);
+  // This is for backwards compat only. Callers should just use the prompt service directly.
+  nsresult rv;
+  nsCOMPtr<nsIPromptFactory> factory = do_GetService("@mozilla.org/prompter;1", &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+  return factory->GetPrompt(aParent, NS_GET_IID(nsIAuthPrompt), reinterpret_cast<void**>(_retval));
 }
 
 NS_IMETHODIMP
 nsWindowWatcher::GetPrompt(nsIDOMWindow *aParent, const nsIID& aIID,
                            void **_retval)
 {
-  if (aIID.Equals(NS_GET_IID(nsIPrompt)))
-    return NS_NewPrompter(reinterpret_cast<nsIPrompt**>(_retval), aParent);
-  if (aIID.Equals(NS_GET_IID(nsIAuthPrompt)))
-    return NS_NewAuthPrompter(reinterpret_cast<nsIAuthPrompt**>(_retval),
-                              aParent);
-  if (aIID.Equals(NS_GET_IID(nsIAuthPrompt2))) {
-    nsresult rv = NS_NewAuthPrompter2(reinterpret_cast<nsIAuthPrompt2**>
-                                                      (_retval),
-                                      aParent);
-    if (rv == NS_NOINTERFACE) {
-      // Return an wrapped nsIAuthPrompt (if we can)
-      nsCOMPtr<nsIAuthPrompt> prompt;
-      rv = NS_NewAuthPrompter(getter_AddRefs(prompt), aParent);
-      if (NS_SUCCEEDED(rv)) {
-        NS_WrapAuthPrompt(prompt,
-                          reinterpret_cast<nsIAuthPrompt2**>(_retval));
-        if (!*_retval)
-          rv = NS_ERROR_NOT_AVAILABLE;
-      }
-    }
+  // This is for backwards compat only. Callers should just use the prompt service directly.
+  nsresult rv;
+  nsCOMPtr<nsIPromptFactory> factory = do_GetService("@mozilla.org/prompter;1", &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = factory->GetPrompt(aParent, aIID, _retval);
 
-    return rv;
+  // Allow for an embedding implementation to not support nsIAuthPrompt2.
+  if (rv == NS_NOINTERFACE && aIID.Equals(NS_GET_IID(nsIAuthPrompt2))) {
+    nsCOMPtr<nsIAuthPrompt> oldPrompt;
+    rv = factory->GetPrompt(aParent,
+                            NS_GET_IID(nsIAuthPrompt),
+                            getter_AddRefs(oldPrompt));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    NS_WrapAuthPrompt(oldPrompt, reinterpret_cast<nsIAuthPrompt2**>(_retval));
+    if (!*_retval)
+      rv = NS_ERROR_NOT_AVAILABLE;
   }
-
-  return NS_NOINTERFACE;
-}
-
-NS_IMETHODIMP
-nsWindowWatcher::CreateAdapter(nsIAuthPrompt* aPrompt, nsIAuthPrompt2** _retval)
-{
-  *_retval = new AuthPromptWrapper(aPrompt);
-  if (!*_retval)
-    return NS_ERROR_OUT_OF_MEMORY;
-  NS_ADDREF(*_retval);
-  return NS_OK;
+  return rv;
 }
 
 NS_IMETHODIMP
@@ -1161,8 +1154,6 @@ nsWindowWatcher::SetActiveWindow(nsIDOMWindow *aActiveWindow)
 NS_IMETHODIMP
 nsWindowWatcher::AddWindow(nsIDOMWindow *aWindow, nsIWebBrowserChrome *aChrome)
 {
-  nsresult rv;
-
   if (!aWindow)
     return NS_ERROR_INVALID_ARG;
 
@@ -1177,7 +1168,7 @@ nsWindowWatcher::AddWindow(nsIDOMWindow *aWindow, nsIWebBrowserChrome *aChrome)
 
   {
     nsWatcherWindowEntry *info;
-    nsAutoLock lock(mListLock);
+    MutexAutoLock lock(mListLock);
 
     // if we already have an entry for this window, adjust
     // its chrome mapping and return
@@ -1206,13 +1197,12 @@ nsWindowWatcher::AddWindow(nsIDOMWindow *aWindow, nsIWebBrowserChrome *aChrome)
 
   // a window being added to us signifies a newly opened window.
   // send notifications.
-  nsCOMPtr<nsIObserverService> os(do_GetService("@mozilla.org/observer-service;1", &rv));
-  if (os) {
-    nsCOMPtr<nsISupports> domwin(do_QueryInterface(aWindow));
-    rv = os->NotifyObservers(domwin, "domwindowopened", 0);
-  }
+  nsCOMPtr<nsIObserverService> os = mozilla::services::GetObserverService();
+  if (!os)
+    return NS_ERROR_FAILURE;
 
-  return rv;
+  nsCOMPtr<nsISupports> domwin(do_QueryInterface(aWindow));
+  return os->NotifyObservers(domwin, "domwindowopened", 0);
 }
 
 NS_IMETHODIMP
@@ -1275,11 +1265,10 @@ nsresult nsWindowWatcher::RemoveWindow(nsWatcherWindowEntry *inInfo)
 {
   PRUint32  ctr,
             count = mEnumeratorList.Length();
-  nsresult rv;
 
   {
     // notify the enumerators
-    nsAutoLock lock(mListLock);
+    MutexAutoLock lock(mListLock);
     for (ctr = 0; ctr < count; ++ctr) 
       mEnumeratorList[ctr]->WindowRemoved(inInfo);
 
@@ -1291,16 +1280,16 @@ nsresult nsWindowWatcher::RemoveWindow(nsWatcherWindowEntry *inInfo)
 
   // a window being removed from us signifies a newly closed window.
   // send notifications.
-  nsCOMPtr<nsIObserverService> os(do_GetService("@mozilla.org/observer-service;1", &rv));
+  nsCOMPtr<nsIObserverService> os = mozilla::services::GetObserverService();
   if (os) {
 #ifdef USEWEAKREFS
     nsCOMPtr<nsISupports> domwin(do_QueryReferent(inInfo->mWindow));
     if (domwin)
-      rv = os->NotifyObservers(domwin, "domwindowclosed", 0);
+      os->NotifyObservers(domwin, "domwindowclosed", 0);
     // else bummer. since the window is gone, there's nothing to notify with.
 #else
     nsCOMPtr<nsISupports> domwin(do_QueryInterface(inInfo->mWindow));
-    rv = os->NotifyObservers(domwin, "domwindowclosed", 0);
+    os->NotifyObservers(domwin, "domwindowclosed", 0);
 #endif
   }
 
@@ -1315,7 +1304,7 @@ nsWindowWatcher::GetChromeForWindow(nsIDOMWindow *aWindow, nsIWebBrowserChrome *
     return NS_ERROR_INVALID_ARG;
   *_retval = 0;
 
-  nsAutoLock lock(mListLock);
+  MutexAutoLock lock(mListLock);
   nsWatcherWindowEntry *info = FindWindowEntry(aWindow);
   if (info) {
     if (info->mChromeWeak != nsnull) {
@@ -1408,7 +1397,7 @@ nsWindowWatcher::URIfromURL(const char *aURL,
       nsCOMPtr<nsIDocument> doc;
       doc = do_QueryInterface(domDoc);
       if (doc) {
-        baseURI = doc->GetBaseURI();
+        baseURI = doc->GetDocBaseURI();
       }
     }
   }
@@ -1519,8 +1508,6 @@ PRUint32 nsWindowWatcher::CalculateChromeFlags(const char *aFeatures,
                                nsIWebBrowserChrome::CHROME_TOOLBAR);
   NS_CALCULATE_CHROME_FLAG_FOR("location",
                                nsIWebBrowserChrome::CHROME_LOCATIONBAR);
-  NS_CALCULATE_CHROME_FLAG_FOR("directories",
-                               nsIWebBrowserChrome::CHROME_PERSONAL_TOOLBAR);
   NS_CALCULATE_CHROME_FLAG_FOR("personalbar",
                                nsIWebBrowserChrome::CHROME_PERSONAL_TOOLBAR);
   NS_CALCULATE_CHROME_FLAG_FOR("status",
@@ -1545,10 +1532,12 @@ PRUint32 nsWindowWatcher::CalculateChromeFlags(const char *aFeatures,
      to mean "OS' choice." */
 
   // default titlebar and closebox to "on," if not mentioned at all
-  if (!PL_strcasestr(aFeatures, "titlebar"))
-    chromeFlags |= nsIWebBrowserChrome::CHROME_TITLEBAR;
-  if (!PL_strcasestr(aFeatures, "close"))
-    chromeFlags |= nsIWebBrowserChrome::CHROME_WINDOW_CLOSE;
+  if (!(chromeFlags & nsIWebBrowserChrome::CHROME_WINDOW_POPUP)) {
+    if (!PL_strcasestr(aFeatures, "titlebar"))
+      chromeFlags |= nsIWebBrowserChrome::CHROME_TITLEBAR;
+    if (!PL_strcasestr(aFeatures, "close"))
+      chromeFlags |= nsIWebBrowserChrome::CHROME_WINDOW_CLOSE;
+  }
 
   if (aDialog && !presenceFlag)
     chromeFlags = nsIWebBrowserChrome::CHROME_DEFAULT;
@@ -1934,23 +1923,17 @@ nsWindowWatcher::SizeOpenedDocShellItem(nsIDocShellTreeItem *aDocShellItem,
     return;
     
   float devPixelsPerCSSPixel = 1.0;
-  nsCOMPtr<nsIWidget> mainWidget;
-  treeOwnerAsWin->GetMainWidget(getter_AddRefs(mainWidget));
-  if (!mainWidget) {
-    // Some embedding clients don't support nsIDocShellTreeOwner's
-    // GetMainWidget, so try going through nsIBaseWindow's GetParentWidget
-    nsCOMPtr<nsIBaseWindow> shellWindow(do_QueryInterface(aDocShellItem));
-    if (shellWindow)
-      shellWindow->GetParentWidget(getter_AddRefs(mainWidget));
-  }
-  if (mainWidget) {
-    nsCOMPtr<nsIDeviceContext> ctx = mainWidget->GetDeviceContext();
-    /* we might be called by an extension after mainWidget::OnDestroy() */
-    if (ctx) {
-      PRInt32 unitsPerDevPixel = ctx->AppUnitsPerDevPixel();
-      if (unitsPerDevPixel) {
-        devPixelsPerCSSPixel = float(ctx->AppUnitsPerCSSPixel()) /
-                                     unitsPerDevPixel;
+  if (aParent) {
+    nsCOMPtr<nsIDOMDocument> openerDoc;
+    aParent->GetDocument(getter_AddRefs(openerDoc));
+    if (openerDoc) {
+      nsCOMPtr<nsIDocument> doc = do_QueryInterface(openerDoc);
+      nsIPresShell* shell = doc->GetShell();
+      if (shell) {
+        nsPresContext* presContext = shell->GetPresContext();
+        if (presContext) {
+          devPixelsPerCSSPixel = presContext->CSSPixelsToDevPixels(1.0f);
+        }
       }
     }
   }
