@@ -50,7 +50,7 @@ NVIDIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <stdio.h>
 #include <string.h>
 
-#include "compiler/debug.h"
+#include "compiler/compilerdebug.h"
 #include "compiler/preprocessor/slglobals.h"
 
 #undef malloc
@@ -182,12 +182,13 @@ static int AddString(StringTable *stable, const char *s)
     char *str;
 
     len = (int) strlen(s);
-    if (stable->nextFree + len + 1 >= stable->size) {
+    while (stable->nextFree + len + 1 >= stable->size) {
         assert(stable->size < 1000000);
         str = (char *) malloc(stable->size*2);
         memcpy(str, stable->strings, stable->size);
         free(stable->strings);
         stable->strings = str;
+        stable->size = stable->size*2;
     }
     loc = stable->nextFree;
     strcpy(&stable->strings[loc], s);
@@ -334,7 +335,7 @@ static int GrowAtomTable(AtomTable *atable, int size)
             if (newmap)
                 atable->amap = newmap;
             if (newrev)
-                atable->amap = newrev;
+                atable->arev = newrev;
             return -1;
         }
         memset(&newmap[atable->size], 0, (size - atable->size) * sizeof(int));
