@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -44,9 +44,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include "jsstdint.h"
 #include "jsbit.h"
 #include "jsdhash.h"
-#include "jsutil.h"     /* for JS_ASSERT */
+#include "jsutil.h"
+
+using namespace js;
 
 #ifdef JS_DHASHMETER
 # if defined MOZILLA_CLIENT && defined DEBUG_XXXbrendan
@@ -71,16 +74,32 @@
 #define RECURSION_LEVEL(table_) (*(uint32*)(table_->entryStore + \
                                             JS_DHASH_TABLE_SIZE(table_) * \
                                             table_->entrySize))
+/*
+ * Most callers that assert about the recursion level don't care about
+ * this magical value because they are asserting that mutation is
+ * allowed (and therefore the level is 0 or 1, depending on whether they
+ * incremented it).
+ *
+ * Only PL_DHashTableFinish needs to allow this special value.
+ */
+#define IMMUTABLE_RECURSION_LEVEL ((uint32)-1)
+
+#define RECURSION_LEVEL_SAFE_TO_FINISH(table_)                                \
+    (RECURSION_LEVEL(table_) == 0 ||                                          \
+     RECURSION_LEVEL(table_) == IMMUTABLE_RECURSION_LEVEL)
 
 #define ENTRY_STORE_EXTRA                   sizeof(uint32)
-#define INCREMENT_RECURSION_LEVEL(table_)   \
-    JS_BEGIN_MACRO                          \
-      ++RECURSION_LEVEL(table_);            \
+#define INCREMENT_RECURSION_LEVEL(table_)                                     \
+    JS_BEGIN_MACRO                                                            \
+        if (RECURSION_LEVEL(table_) != IMMUTABLE_RECURSION_LEVEL)             \
+            ++RECURSION_LEVEL(table_);                                        \
     JS_END_MACRO
-#define DECREMENT_RECURSION_LEVEL(table_)                  \
-    JS_BEGIN_MACRO                                         \
-      JSDHASH_ONELINE_ASSERT(RECURSION_LEVEL(table_) > 0); \
-      --RECURSION_LEVEL(table_);                           \
+#define DECREMENT_RECURSION_LEVEL(table_)                                     \
+    JS_BEGIN_MACRO                                                            \
+        if (RECURSION_LEVEL(table_) != IMMUTABLE_RECURSION_LEVEL) {           \
+            JSDHASH_ONELINE_ASSERT(RECURSION_LEVEL(table_) > 0);              \
+            --RECURSION_LEVEL(table_);                                        \
+        }                                                                     \
     JS_END_MACRO
 
 #else
@@ -94,13 +113,13 @@
 JS_PUBLIC_API(void *)
 JS_DHashAllocTable(JSDHashTable *table, uint32 nbytes)
 {
-    return malloc(nbytes);
+    return OffTheBooks::malloc_(nbytes);
 }
 
 JS_PUBLIC_API(void)
 JS_DHashFreeTable(JSDHashTable *table, void *ptr)
 {
-    free(ptr);
+    UnwantedForeground::free_(ptr);
 }
 
 JS_PUBLIC_API(JSDHashNumber)
@@ -118,7 +137,7 @@ JS_DHashStringKey(JSDHashTable *table, const void *key)
 JS_PUBLIC_API(JSDHashNumber)
 JS_DHashVoidPtrKeyStub(JSDHashTable *table, const void *key)
 {
-    return (JSDHashNumber)(unsigned long)key >> 2;
+    return (JSDHashNumber)(uintptr_t)key >> 2;
 }
 
 JS_PUBLIC_API(JSBool)
@@ -163,7 +182,7 @@ JS_DHashFreeStringKey(JSDHashTable *table, JSDHashEntryHdr *entry)
 {
     const JSDHashEntryStub *stub = (const JSDHashEntryStub *)entry;
 
-    free((void *) stub->key);
+    UnwantedForeground::free_((void *) stub->key);
     memset(entry, 0, table->entrySize);
 }
 
@@ -195,11 +214,11 @@ JS_NewDHashTable(const JSDHashTableOps *ops, void *data, uint32 entrySize,
 {
     JSDHashTable *table;
 
-    table = (JSDHashTable *) malloc(sizeof *table);
+    table = (JSDHashTable *) OffTheBooks::malloc_(sizeof *table);
     if (!table)
         return NULL;
     if (!JS_DHashTableInit(table, ops, data, entrySize, capacity)) {
-        free(table);
+        Foreground::free_(table);
         return NULL;
     }
     return table;
@@ -209,7 +228,7 @@ JS_PUBLIC_API(void)
 JS_DHashTableDestroy(JSDHashTable *table)
 {
     JS_DHashTableFinish(table);
-    free(table);
+    UnwantedForeground::free_(table);
 }
 
 JS_PUBLIC_API(JSBool)
@@ -382,7 +401,7 @@ JS_DHashTableFinish(JSDHashTable *table)
     }
 
     DECREMENT_RECURSION_LEVEL(table);
-    JS_ASSERT(RECURSION_LEVEL(table) == 0);
+    JS_ASSERT(RECURSION_LEVEL_SAFE_TO_FINISH(table));
 
     /* Free entry storage last. */
     table->ops->freeTable(table, table->entryStore);
@@ -688,6 +707,8 @@ JS_DHashTableRawRemove(JSDHashTable *table, JSDHashEntryHdr *entry)
 {
     JSDHashNumber keyHash;      /* load first in case clearEntry goofs it */
 
+    JS_ASSERT(RECURSION_LEVEL(table) != IMMUTABLE_RECURSION_LEVEL);
+
     JS_ASSERT(JS_DHASH_ENTRY_IS_LIVE(entry));
     keyHash = entry->keyHash;
     table->ops->clearEntry(table, entry);
@@ -763,6 +784,14 @@ JS_DHashTableEnumerate(JSDHashTable *table, JSDHashEnumerator etor, void *arg)
     return i;
 }
 
+#ifdef DEBUG
+JS_PUBLIC_API(void)
+JS_DHashMarkTableImmutable(JSDHashTable *table)
+{
+    RECURSION_LEVEL(table) = IMMUTABLE_RECURSION_LEVEL;
+}
+#endif
+
 #ifdef JS_DHASHMETER
 #include <math.h>
 
