@@ -38,27 +38,26 @@
 
 
 /**
-   File Name:          7.3-2.js
-   ECMA Section:       7.3 Comments
-   Description:
+   File Name:          7.7.1.js
+   ECMA Section:       7.7.1 Null Literals
 
+   Description:        NullLiteral::
+   null
+
+
+   The value of the null literal null is the sole value
+   of the Null type, namely null.
 
    Author:             christine@netscape.com
-   Date:               12 november 1997
-
+   Date:               21 october 1997
 */
-var SECTION = "7.3-2";
+var SECTION = "7.7.1";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "Comments";
+var TITLE   = "Null Literals";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-var testcase = new TestCase( SECTION,
-			     "a comment with a carriage return, and text following",
-			     "pass",
-			     "pass");
-
-// "\u000D" testcase.actual = "fail";
+new TestCase( SECTION, "null",     null,        null);
 
 test();
