@@ -38,37 +38,61 @@
 
 
 /**
-   File Name:          12.6.2-6.js
-   ECMA Section:       12.6.2 The for Statement
+   File Name:          12.5-2.js
+   ECMA Section:       The if statement
+   Description:
 
-   1. first expression is present.
-   2. second expression is not present
-   3. third expression is present
+   The production IfStatement : if ( Expression ) Statement else Statement
+   is evaluated as follows:
 
+   1.Evaluate Expression.
+   2.Call GetValue(Result(1)).
+   3.Call ToBoolean(Result(2)).
+   4.If Result(3) is false, go to step 7.
+   5.Evaluate the first Statement.
+   6.Return Result(5).
+   7.Evaluate the second Statement.
+   8.Return Result(7).
 
    Author:             christine@netscape.com
-   Date:               15 september 1997
+   Date:               12 november 1997
 */
-var SECTION = "12.6.2-6";
+
+var SECTION = "12.5-2";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "The for statement";
+var TITLE = "The if statement" ;
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-new TestCase( "12.6.2-6", "for statement",  256,     testprogram() );
+new TestCase(   SECTION,
+		"var MYVAR; if ( true ) MYVAR='PASSED'; MYVAR",
+		"PASSED",
+		eval("var MYVAR; if ( true ) MYVAR='PASSED'; MYVAR") );
+
+new TestCase(  SECTION,
+	       "var MYVAR; if ( false ) MYVAR='FAILED'; MYVAR;",
+	       "PASSED",
+	       eval("var MYVAR=\"PASSED\"; if ( false ) MYVAR='FAILED'; MYVAR;") );
+
+new TestCase(   SECTION,
+		"var MYVAR; if ( new Boolean(true) ) MYVAR='PASSED'; MYVAR",
+		"PASSED",
+		eval("var MYVAR; if ( new Boolean(true) ) MYVAR='PASSED'; MYVAR") );
+
+new TestCase(   SECTION,
+		"var MYVAR; if ( new Boolean(false) ) MYVAR='PASSED'; MYVAR",
+		"PASSED",
+		eval("var MYVAR; if ( new Boolean(false) ) MYVAR='PASSED'; MYVAR") );
+
+new TestCase(   SECTION,
+		"var MYVAR; if ( 1 ) MYVAR='PASSED'; MYVAR",
+		"PASSED",
+		eval("var MYVAR; if ( 1 ) MYVAR='PASSED'; MYVAR") );
+
+new TestCase(  SECTION,
+	       "var MYVAR; if ( 0 ) MYVAR='FAILED'; MYVAR;",
+	       "PASSED",
+	       eval("var MYVAR=\"PASSED\"; if ( 0 ) MYVAR='FAILED'; MYVAR;") );
 
 test();
-
-function testprogram() {
-  var myVar;
-
-  for ( myVar=2; ; myVar *= myVar ) {
-
-    if (myVar > 100)
-      break;
-    continue;
-  }
-
-  return myVar;
-}
