@@ -53,10 +53,11 @@
 #include "nsIDOMWindowInternal.h"
 #include "nsXBLBinding.h"
 #include "nsXBLPrototypeBinding.h"
-#include "nsIDOMElement.h"
 #include "nsIMutableArray.h"
 #include "nsBindingManager.h"
 #include "nsComputedDOMStyle.h"
+#include "nsEventStateManager.h"
+#include "nsIAtom.h"
 
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -150,16 +151,22 @@ inDOMUtils::GetParentForNode(nsIDOMNode* aNode,
 
 NS_IMETHODIMP
 inDOMUtils::GetCSSStyleRules(nsIDOMElement *aElement,
+                             const nsAString& aPseudo,
                              nsISupportsArray **_retval)
 {
   NS_ENSURE_ARG_POINTER(aElement);
 
   *_retval = nsnull;
 
+  nsCOMPtr<nsIAtom> pseudoElt;
+  if (!aPseudo.IsEmpty()) {
+    pseudoElt = do_GetAtom(aPseudo);
+  }
+
   nsRuleNode* ruleNode = nsnull;
   nsCOMPtr<nsIContent> content = do_QueryInterface(aElement);
   nsRefPtr<nsStyleContext> styleContext;
-  GetRuleNodeForContent(content, getter_AddRefs(styleContext), &ruleNode);
+  GetRuleNodeForContent(content, pseudoElt, getter_AddRefs(styleContext), &ruleNode);
   if (!ruleNode) {
     // This can fail for content nodes that are not in the document or
     // if the document they're in doesn't have a presshell.  Bail out.
@@ -171,11 +178,10 @@ inDOMUtils::GetCSSStyleRules(nsIDOMElement *aElement,
   if (!rules) return NS_ERROR_OUT_OF_MEMORY;
 
   nsRefPtr<mozilla::css::StyleRule> cssRule;
-  nsCOMPtr<nsIDOMCSSRule> domRule;
   for ( ; !ruleNode->IsRoot(); ruleNode = ruleNode->GetParent()) {
     cssRule = do_QueryObject(ruleNode->GetRule());
     if (cssRule) {
-      cssRule->GetDOMRule(getter_AddRefs(domRule));
+      nsCOMPtr<nsIDOMCSSRule> domRule = cssRule->GetDOMRule();
       if (domRule)
         rules->InsertElementAt(domRule, 0);
     }
@@ -236,7 +242,7 @@ inDOMUtils::SetContentState(nsIDOMElement *aElement, nsEventStates::InternalType
 {
   NS_ENSURE_ARG_POINTER(aElement);
   
-  nsCOMPtr<nsIEventStateManager> esm = inLayoutUtils::GetEventStateManagerFor(aElement);
+  nsRefPtr<nsEventStateManager> esm = inLayoutUtils::GetEventStateManagerFor(aElement);
   if (esm) {
     nsCOMPtr<nsIContent> content;
     content = do_QueryInterface(aElement);
@@ -254,7 +260,7 @@ inDOMUtils::GetContentState(nsIDOMElement *aElement, nsEventStates::InternalType
 
   NS_ENSURE_ARG_POINTER(aElement);
 
-  nsCOMPtr<nsIEventStateManager> esm = inLayoutUtils::GetEventStateManagerFor(aElement);
+  nsRefPtr<nsEventStateManager> esm = inLayoutUtils::GetEventStateManagerFor(aElement);
   if (esm) {
     nsCOMPtr<nsIContent> content;
     content = do_QueryInterface(aElement);
@@ -269,6 +275,7 @@ inDOMUtils::GetContentState(nsIDOMElement *aElement, nsEventStates::InternalType
 
 /* static */ nsresult
 inDOMUtils::GetRuleNodeForContent(nsIContent* aContent,
+                                  nsIAtom* aPseudo,
                                   nsStyleContext** aStyleContext,
                                   nsRuleNode** aRuleNode)
 {
@@ -292,9 +299,10 @@ inDOMUtils::GetRuleNodeForContent(nsIContent* aContent,
   NS_ENSURE_TRUE(safe, NS_ERROR_OUT_OF_MEMORY);
 
   nsRefPtr<nsStyleContext> sContext =
-    nsComputedDOMStyle::GetStyleContextForElement(aContent->AsElement(),
-						  nsnull, presShell);
-  *aRuleNode = sContext->GetRuleNode();
-  sContext.forget(aStyleContext);
+    nsComputedDOMStyle::GetStyleContextForElement(aContent->AsElement(), aPseudo, presShell);
+  if (sContext) {
+    *aRuleNode = sContext->GetRuleNode();
+    sContext.forget(aStyleContext);
+  }
   return NS_OK;
 }
