@@ -68,7 +68,6 @@
 // MACRO_NAME=CSS_KEY
 
 CSS_KEY(-moz-activehyperlinktext, _moz_activehyperlinktext)
-CSS_KEY(-moz-alias, _moz_alias)
 CSS_KEY(-moz-all, _moz_all)
 CSS_KEY(-moz-alt-content, _moz_alt_content)
 CSS_KEY(-moz-anchor-decoration, _moz_anchor_decoration)
@@ -80,23 +79,24 @@ CSS_KEY(-moz-button, _moz_button)
 CSS_KEY(-moz-buttondefault, _moz_buttondefault)
 CSS_KEY(-moz-buttonhoverface, _moz_buttonhoverface)
 CSS_KEY(-moz-buttonhovertext, _moz_buttonhovertext)
-CSS_KEY(-moz-cell, _moz_cell)
 CSS_KEY(-moz-cellhighlight, _moz_cellhighlight)
 CSS_KEY(-moz-cellhighlighttext, _moz_cellhighlighttext)
 CSS_KEY(-moz-center, _moz_center)
 CSS_KEY(-moz-cjk-earthly-branch, _moz_cjk_earthly_branch)
 CSS_KEY(-moz-cjk-heavenly-stem, _moz_cjk_heavenly_stem)
-CSS_KEY(-moz-compact, _moz_compact)  // Disabled because not supported.
+CSS_KEY(-moz-combobox, _moz_combobox)
+CSS_KEY(-moz-comboboxtext, _moz_comboboxtext)
 CSS_KEY(-moz-block-height, _moz_block_height)
-CSS_KEY(-moz-context-menu, _moz_context_menu)
-CSS_KEY(-moz-copy, _moz_copy)
 CSS_KEY(-moz-deck, _moz_deck)
+CSS_KEY(-moz-default-background-color, _moz_default_background_color)
+CSS_KEY(-moz-default-color, _moz_default_color)
 CSS_KEY(-moz-desktop, _moz_desktop)
 CSS_KEY(-moz-devanagari, _moz_devanagari)
 CSS_KEY(-moz-dialog, _moz_dialog)
 CSS_KEY(-moz-dialogtext, _moz_dialogtext)
 CSS_KEY(-moz-document, _moz_document)
 CSS_KEY(-moz-dragtargetzone, _moz_dragtargetzone)
+CSS_KEY(-moz-element, _moz_element)
 CSS_KEY(-moz-eventreerow, _moz_eventreerow)
 CSS_KEY(-moz-ethiopic-halehame, _moz_ethiopic_halehame)
 CSS_KEY(-moz-ethiopic-numeric, _moz_ethiopic_numeric)
@@ -120,6 +120,7 @@ CSS_KEY(-moz-hangul, _moz_hangul)
 CSS_KEY(-moz-hyperlinktext, _moz_hyperlinktext)
 CSS_KEY(-moz-html-cellhighlight, _moz_html_cellhighlight)
 CSS_KEY(-moz-html-cellhighlighttext, _moz_html_cellhighlighttext)
+CSS_KEY(-moz-image-rect, _moz_image_rect)
 CSS_KEY(-moz-info, _moz_info)
 CSS_KEY(-moz-initial, _moz_initial)
 CSS_KEY(-moz-inline-box, _moz_inline_box)
@@ -132,13 +133,6 @@ CSS_KEY(-moz-khmer, _moz_khmer)
 CSS_KEY(-moz-lao, _moz_lao)
 CSS_KEY(-moz-left, _moz_left)
 CSS_KEY(-moz-list, _moz_list)
-CSS_KEY(-moz-mac-accentdarkestshadow, _moz_mac_accentdarkestshadow)
-CSS_KEY(-moz-mac-accentdarkshadow, _moz_mac_accentdarkshadow)
-CSS_KEY(-moz-mac-accentface, _moz_mac_accentface)
-CSS_KEY(-moz-mac-accentlightesthighlight, _moz_mac_accentlightesthighlight)
-CSS_KEY(-moz-mac-accentlightshadow, _moz_mac_accentlightshadow)
-CSS_KEY(-moz-mac-accentregularhighlight, _moz_mac_accentregularhighlight)
-CSS_KEY(-moz-mac-accentregularshadow, _moz_mac_accentregularshadow)
 CSS_KEY(-moz-mac-alternateprimaryhighlight, _moz_mac_alternateprimaryhighlight)
 CSS_KEY(-moz-mac-chrome-active, _moz_mac_chrome_active)
 CSS_KEY(-moz-mac-chrome-inactive, _moz_mac_chrome_inactive)
@@ -150,10 +144,10 @@ CSS_KEY(-moz-mac-menutextselect, _moz_mac_menutextselect)
 CSS_KEY(-moz-mac-disabledtoolbartext, _moz_mac_disabledtoolbartext)
 CSS_KEY(-moz-mac-secondaryhighlight, _moz_mac_secondaryhighlight)
 CSS_KEY(-moz-malayalam, _moz_malayalam)
-CSS_KEY(-moz-marker, _moz_marker) // Disabled because not supported correctly.
 CSS_KEY(-moz-max-content, _moz_max_content)
 CSS_KEY(-moz-menuhover, _moz_menuhover)
 CSS_KEY(-moz-menuhovertext, _moz_menuhovertext)
+CSS_KEY(-moz-menubartext, _moz_menubartext)
 CSS_KEY(-moz-menubarhovertext, _moz_menubarhovertext)
 CSS_KEY(-moz-middle-with-baseline, _moz_middle_with_baseline)
 CSS_KEY(-moz-min-content, _moz_min_content)
@@ -166,14 +160,12 @@ CSS_KEY(-moz-persian, _moz_persian)
 CSS_KEY(-moz-popup, _moz_popup)
 CSS_KEY(-moz-pull-down-menu, _moz_pull_down_menu)
 CSS_KEY(-moz-right, _moz_right)
-CSS_KEY(-moz-run-in, _moz_run_in) // Disabled because not supported.
 CSS_KEY(-moz-scrollbars-horizontal, _moz_scrollbars_horizontal)
 CSS_KEY(-moz-scrollbars-none, _moz_scrollbars_none)
 CSS_KEY(-moz-scrollbars-vertical, _moz_scrollbars_vertical)
 CSS_KEY(-moz-show-background, _moz_show_background)
 CSS_KEY(-moz-simp-chinese-formal, _moz_simp_chinese_formal)
 CSS_KEY(-moz-simp-chinese-informal, _moz_simp_chinese_informal)
-CSS_KEY(-moz-spinning, _moz_spinning)
 CSS_KEY(-moz-stack, _moz_stack)
 CSS_KEY(-moz-tamil, _moz_tamil)
 CSS_KEY(-moz-telugu, _moz_telugu)
@@ -188,7 +180,6 @@ CSS_KEY(-moz-window, _moz_window)
 CSS_KEY(-moz-workspace, _moz_workspace)
 CSS_KEY(-moz-zoom-in, _moz_zoom_in)
 CSS_KEY(-moz-zoom-out, _moz_zoom_out)
-CSS_KEY(above, above)
 CSS_KEY(absolute, absolute)
 CSS_KEY(active, active)
 CSS_KEY(activeborder, activeborder)
@@ -196,22 +187,21 @@ CSS_KEY(activecaption, activecaption)
 CSS_KEY(alias, alias)
 CSS_KEY(all, all)
 CSS_KEY(all-scroll, all_scroll)
+CSS_KEY(alternate, alternate)
 CSS_KEY(always, always)
 CSS_KEY(appworkspace, appworkspace)
 CSS_KEY(armenian, armenian)
 CSS_KEY(auto, auto)
 CSS_KEY(avoid, avoid)
 CSS_KEY(background, background)
+CSS_KEY(backwards, backwards)
 CSS_KEY(baseline, baseline)
-CSS_KEY(behind, behind)
-CSS_KEY(below, below)
 CSS_KEY(bidi-override, bidi_override)
 CSS_KEY(blink, blink)
 CSS_KEY(block, block)
 CSS_KEY(block-axis, block_axis)
 CSS_KEY(bold, bold)
 CSS_KEY(bolder, bolder)
-CSS_KEY(border, border)
 CSS_KEY(border-box, border_box)
 CSS_KEY(both, both)
 CSS_KEY(bottom, bottom)
@@ -228,22 +218,22 @@ CSS_KEY(caption, caption)
 CSS_KEY(captiontext, captiontext)
 CSS_KEY(cell, cell)
 CSS_KEY(center, center)
-CSS_KEY(center-left, center_left)
-CSS_KEY(center-right, center_right)
 CSS_KEY(ch, ch)
 CSS_KEY(circle, circle)
 CSS_KEY(cjk-ideographic, cjk_ideographic)
 CSS_KEY(close-quote, close_quote)
+CSS_KEY(closest-corner, closest_corner)
+CSS_KEY(closest-side, closest_side)
 CSS_KEY(cm, cm)
-CSS_KEY(code, code)
 CSS_KEY(col-resize, col_resize)
 CSS_KEY(collapse, collapse)
 CSS_KEY(condensed, condensed)
-CSS_KEY(content, content)
+CSS_KEY(contain, contain)
 CSS_KEY(content-box, content_box)
 CSS_KEY(context-menu, context_menu)
 CSS_KEY(continuous, continuous)
 CSS_KEY(copy, copy)
+CSS_KEY(cover, cover)
 CSS_KEY(crop, crop)
 CSS_KEY(cross, cross)
 CSS_KEY(crosshair, crosshair)
@@ -254,15 +244,19 @@ CSS_KEY(decimal-leading-zero, decimal_leading_zero)
 CSS_KEY(default, default)
 CSS_KEY(deg, deg)
 CSS_KEY(dialog, dialog)
-CSS_KEY(digits, digits)
 CSS_KEY(disabled, disabled)
 CSS_KEY(disc, disc)
 CSS_KEY(dotted, dotted)
 CSS_KEY(double, double)
 CSS_KEY(e-resize, e_resize)
 CSS_KEY(each-box, each_box)
+CSS_KEY(ease, ease)
+CSS_KEY(ease-in, ease_in)
+CSS_KEY(ease-in-out, ease_in_out)
+CSS_KEY(ease-out, ease_out)
 CSS_KEY(element, element)
 CSS_KEY(elements, elements)
+CSS_KEY(ellipse, ellipse)
 CSS_KEY(em, em)
 CSS_KEY(embed, embed)
 CSS_KEY(enabled, enabled)
@@ -272,11 +266,11 @@ CSS_KEY(expanded, expanded)
 CSS_KEY(extra-condensed, extra_condensed)
 CSS_KEY(extra-expanded, extra_expanded)
 CSS_KEY(ew-resize, ew_resize)
-CSS_KEY(far-left, far_left)
-CSS_KEY(far-right, far_right)
-CSS_KEY(fast, fast)
-CSS_KEY(faster, faster)
+CSS_KEY(farthest-side, farthest_side)
+CSS_KEY(farthest-corner, farthest_corner)
+CSS_KEY(fill, fill)
 CSS_KEY(fixed, fixed)
+CSS_KEY(forwards, forwards)
 CSS_KEY(georgian, georgian)
 CSS_KEY(grad, grad)
 CSS_KEY(graytext, graytext)
@@ -285,8 +279,6 @@ CSS_KEY(hebrew, hebrew)
 CSS_KEY(help, help)
 CSS_KEY(hidden, hidden)
 CSS_KEY(hide, hide)
-CSS_KEY(high, high)
-CSS_KEY(higher, higher)
 CSS_KEY(highlight, highlight)
 CSS_KEY(highlighttext, highlighttext)
 CSS_KEY(hiragana, hiragana)
@@ -301,6 +293,7 @@ CSS_KEY(inactive, inactive)
 CSS_KEY(inactiveborder, inactiveborder)
 CSS_KEY(inactivecaption, inactivecaption)
 CSS_KEY(inactivecaptiontext, inactivecaptiontext)
+CSS_KEY(infinite, infinite)
 CSS_KEY(infobackground, infobackground)
 CSS_KEY(infotext, infotext)
 CSS_KEY(inherit, inherit)
@@ -322,22 +315,18 @@ CSS_KEY(landscape, landscape)
 CSS_KEY(large, large)
 CSS_KEY(larger, larger)
 CSS_KEY(left, left)
-CSS_KEY(left-side, left_side)
-CSS_KEY(leftwards, leftwards)
-CSS_KEY(level, level)
 CSS_KEY(lighter, lighter)
 CSS_KEY(line-through, line_through)
+CSS_KEY(linear, linear)
 CSS_KEY(list-item, list_item)
 CSS_KEY(logical, logical)
-CSS_KEY(loud, loud)
-CSS_KEY(low, low)
-CSS_KEY(lower, lower)
 CSS_KEY(lower-alpha, lower_alpha)
 CSS_KEY(lower-greek, lower_greek)
 CSS_KEY(lower-latin, lower_latin)
 CSS_KEY(lower-roman, lower_roman)
 CSS_KEY(lowercase, lowercase)
 CSS_KEY(ltr, ltr)
+CSS_KEY(manual, manual)
 CSS_KEY(margin-box, margin_box)
 CSS_KEY(matrix, matrix)
 CSS_KEY(medium, medium)
@@ -365,13 +354,13 @@ CSS_KEY(ns-resize, ns_resize)
 CSS_KEY(nw-resize, nw_resize)
 CSS_KEY(nwse-resize, nwse_resize)
 CSS_KEY(oblique, oblique)
-CSS_KEY(once, once)
 CSS_KEY(open-quote, open_quote)
 CSS_KEY(outset, outset)
 CSS_KEY(outside, outside)
 CSS_KEY(overline, overline)
-CSS_KEY(padding, padding)
 CSS_KEY(padding-box, padding_box)
+CSS_KEY(painted, painted)
+CSS_KEY(paused, paused)
 CSS_KEY(pc, pc)
 CSS_KEY(physical, physical)
 CSS_KEY(pointer, pointer)
@@ -393,12 +382,11 @@ CSS_KEY(repeat-y, repeat_y)
 CSS_KEY(reverse, reverse)
 CSS_KEY(ridge, ridge)
 CSS_KEY(right, right)
-CSS_KEY(right-side, right_side)
-CSS_KEY(rightwards, rightwards)
 CSS_KEY(rotate, rotate)
 CSS_KEY(round, round)
 CSS_KEY(row-resize, row_resize)
 CSS_KEY(rtl, rtl)
+CSS_KEY(running, running)
 CSS_KEY(s, s)
 CSS_KEY(s-resize, s_resize)
 CSS_KEY(scale, scale)
@@ -417,25 +405,24 @@ CSS_KEY(semi-condensed, semi_condensed)
 CSS_KEY(semi-expanded, semi_expanded)
 CSS_KEY(separate, separate)
 CSS_KEY(show, show)
-CSS_KEY(silent, silent)
 CSS_KEY(skew, skew)
 CSS_KEY(skewx, skewx)
 CSS_KEY(skewy, skewy)
-CSS_KEY(slow, slow)
-CSS_KEY(slower, slower)
 CSS_KEY(small, small)
 CSS_KEY(small-caps, small_caps)
 CSS_KEY(small-caption, small_caption)
 CSS_KEY(smaller, smaller)
 CSS_KEY(soft, soft)
 CSS_KEY(solid, solid)
-CSS_KEY(spell-out, spell_out)
 CSS_KEY(square, square)
 CSS_KEY(start, start)
 CSS_KEY(static, static)
 CSS_KEY(status-bar, status_bar)
+CSS_KEY(step-end, step_end)
+CSS_KEY(step-start, step_start)
 CSS_KEY(stretch, stretch)
 CSS_KEY(stretch-to-fit, stretch_to_fit)
+CSS_KEY(stroke, stroke)
 CSS_KEY(sub, sub)
 CSS_KEY(super, super)
 CSS_KEY(sw-resize, sw_resize)
@@ -464,6 +451,7 @@ CSS_KEY(top-outside, top_outside)
 CSS_KEY(translate, translate)
 CSS_KEY(translatex, translatex)
 CSS_KEY(translatey, translatey)
+CSS_KEY(transparent, transparent) // for nsComputedDOMStyle only
 CSS_KEY(tri-state, tri_state)
 CSS_KEY(ultra-condensed, ultra_condensed)
 CSS_KEY(ultra-expanded, ultra_expanded)
@@ -475,21 +463,19 @@ CSS_KEY(uppercase, uppercase)
 CSS_KEY(vertical, vertical)
 CSS_KEY(vertical-text, vertical_text)
 CSS_KEY(visible, visible)
+CSS_KEY(visiblefill, visiblefill)
+CSS_KEY(visiblepainted, visiblepainted)
+CSS_KEY(visiblestroke, visiblestroke)
 CSS_KEY(w-resize, w_resize)
 CSS_KEY(wait, wait)
+CSS_KEY(wavy, wavy)
 CSS_KEY(wider, wider)
 CSS_KEY(window, window)
 CSS_KEY(windowframe, windowframe)
 CSS_KEY(windowtext, windowtext)
 CSS_KEY(write-only, write_only)
-CSS_KEY(x-fast, x_fast)
-CSS_KEY(x-high, x_high)
 CSS_KEY(x-large, x_large)
-CSS_KEY(x-loud, x_loud)
-CSS_KEY(x-low, x_low)
-CSS_KEY(x-slow, x_slow)
 CSS_KEY(x-small, x_small)
-CSS_KEY(x-soft, x_soft)
 CSS_KEY(xx-large, xx_large)
 CSS_KEY(xx-small, xx_small)
 
@@ -503,6 +489,10 @@ CSS_KEY(toolbarbutton, toolbarbutton)
 CSS_KEY(toolbargripper, toolbargripper)
 CSS_KEY(dualbutton, dualbutton)
 CSS_KEY(toolbarbutton-dropdown, toolbarbutton_dropdown)
+CSS_KEY(button-arrow-up, button_arrow_up)
+CSS_KEY(button-arrow-down, button_arrow_down)
+CSS_KEY(button-arrow-next, button_arrow_next)
+CSS_KEY(button-arrow-previous, button_arrow_previous)
 CSS_KEY(separator, separator)
 CSS_KEY(splitter, splitter)
 CSS_KEY(statusbar, statusbar)
@@ -524,8 +514,6 @@ CSS_KEY(progressbar-vertical, progressbar_vertical)
 CSS_KEY(progresschunk, progresschunk)
 CSS_KEY(progresschunk-vertical, progresschunk_vertical)
 CSS_KEY(tab, tab)
-CSS_KEY(tab-left-edge, tab_left_edge)
-CSS_KEY(tab-right-edge, tab_right_edge)
 CSS_KEY(tabpanels, tabpanels)
 CSS_KEY(tabpanel, tabpanel)
 CSS_KEY(tab-scroll-arrow-back, tabscrollarrow_back)
@@ -543,6 +531,7 @@ CSS_KEY(scrollbartrack-horizontal, scrollbartrack_horizontal)
 CSS_KEY(scrollbartrack-vertical, scrollbartrack_vertical)
 CSS_KEY(scrollbarthumb-horizontal, scrollbarthumb_horizontal)
 CSS_KEY(scrollbarthumb-vertical, scrollbarthumb_vertical)
+CSS_KEY(sheet, sheet)
 CSS_KEY(textfield, textfield)
 CSS_KEY(textfield-multiline, textfield_multiline)
 CSS_KEY(caret, caret)
@@ -581,19 +570,27 @@ CSS_KEY(-moz-win-browsertabbar-toolbox, _moz_win_browsertabbar_toolbox)
 CSS_KEY(-moz-win-mediatext, _moz_win_mediatext)
 CSS_KEY(-moz-win-communicationstext, _moz_win_communicationstext)
 CSS_KEY(-moz-win-glass, _moz_win_glass)
+CSS_KEY(-moz-win-borderless-glass, _moz_win_borderless_glass)
+CSS_KEY(-moz-window-titlebar, _moz_window_titlebar)
+CSS_KEY(-moz-window-titlebar-maximized, _moz_window_titlebar_maximized)
+CSS_KEY(-moz-window-frame-left, _moz_window_frame_left)
+CSS_KEY(-moz-window-frame-right, _moz_window_frame_right)
+CSS_KEY(-moz-window-frame-bottom, _moz_window_frame_bottom)
+CSS_KEY(-moz-window-button-close, _moz_window_button_close)
+CSS_KEY(-moz-window-button-minimize, _moz_window_button_minimize)
+CSS_KEY(-moz-window-button-maximize, _moz_window_button_maximize)
+CSS_KEY(-moz-window-button-restore, _moz_window_button_restore)
+CSS_KEY(-moz-window-button-box, _moz_window_button_box)
+CSS_KEY(-moz-window-button-box-maximized, _moz_window_button_box_maximized)
 CSS_KEY(-moz-mac-unified-toolbar, _moz_mac_unified_toolbar)
-
-#ifdef MOZ_SVG
-//CSS_KEY(all, all)
+CSS_KEY(-moz-win-exclude-glass, _moz_win_exclude_glass)
 CSS_KEY(alphabetic, alphabetic)
-//CSS_KEY(auto, auto)
 CSS_KEY(bevel, bevel)
 CSS_KEY(butt, butt)
 CSS_KEY(central, central)
 CSS_KEY(crispedges, crispedges)
 //CSS_KEY(end, end)
 CSS_KEY(evenodd, evenodd)
-CSS_KEY(fill, fill)
 CSS_KEY(geometricprecision, geometricprecision)
 CSS_KEY(hanging, hanging)
 CSS_KEY(ideographic, ideographic)
@@ -602,21 +599,15 @@ CSS_KEY(mathematical, mathematical)
 //CSS_KEY(middle, middle)
 CSS_KEY(miter, miter)
 CSS_KEY(no-change, no_change)
-//CSS_KEY(none, none)
 CSS_KEY(nonzero, nonzero)
 CSS_KEY(optimizelegibility, optimizelegibility)
+CSS_KEY(optimizequality, optimizequality)
 CSS_KEY(optimizespeed, optimizespeed)
-CSS_KEY(painted, painted)
 CSS_KEY(reset-size, reset_size)
 //CSS_KEY(square, square)
 //CSS_KEY(start, start)
 CSS_KEY(srgb, srgb)
-CSS_KEY(stroke, stroke)
 CSS_KEY(text-after-edge, text_after_edge)
 CSS_KEY(text-before-edge, text_before_edge)
 CSS_KEY(use-script, use_script)
-//CSS_KEY(visible, visible)
-CSS_KEY(visiblefill, visiblefill)
-CSS_KEY(visiblepainted, visiblepainted)
-CSS_KEY(visiblestroke, visiblestroke)
-#endif
+CSS_KEY(-moz-crisp-edges, _moz_crisp_edges)
