@@ -246,10 +246,12 @@ png_text_compress(png_structp png_ptr,
     */
 
    /* Set up the compression buffers */
+   /* TODO: the following cast hides a potential overflow problem. */
    png_ptr->zstream.avail_in = (uInt)text_len;
+   /* NOTE: assume zlib doesn't overwrite the input */
    png_ptr->zstream.next_in = (Bytef *)text;
-   png_ptr->zstream.avail_out = (uInt)png_ptr->zbuf_size;
-   png_ptr->zstream.next_out = (Bytef *)png_ptr->zbuf;
+   png_ptr->zstream.avail_out = png_ptr->zbuf_size;
+   png_ptr->zstream.next_out = png_ptr->zbuf;
 
    /* This is the same compression loop as in png_write_row() */
    do
@@ -688,11 +690,29 @@ png_write_IDAT(png_structp png_ptr, png_bytep data, png_size_t length)
          if (length >= 2 &&
              png_ptr->height < 16384 && png_ptr->width < 16384)
          {
+            unsigned int z_cinfo;
+            unsigned int half_z_window_size;
+
+            /* Compute the maximum possible length of the datastream */
+
+            /* Number of pixels, plus for each row a filter byte and possible
+             * and possibly a padding byte, so increase the maximum
+             * size to account for these.
+             */
             png_uint_32 uncompressed_idat_size = png_ptr->height *
                ((png_ptr->width *
                png_ptr->channels * png_ptr->bit_depth + 15) >> 3);
-            unsigned int z_cinfo = z_cmf >> 4;
-            unsigned int half_z_window_size = 1 << (z_cinfo + 7);
+
+            /* If it's interlaced, each block of 8 rows is sent as up to
+             * 14 rows, i.e., 6 additional rows, each with a filter byte
+             * and possibly a padding byte
+             */
+            if (png_ptr->interlaced)
+               uncompressed_idat_size += ((png_ptr->height + 7)/8) *
+                   (png_ptr->bit_depth < 8 ? 12 : 6);
+
+            z_cinfo = z_cmf >> 4;
+            half_z_window_size = 1 << (z_cinfo + 7);
             while (uncompressed_idat_size <= half_z_window_size &&
                    half_z_window_size >= 256)
             {
@@ -721,16 +741,16 @@ png_write_IDAT(png_structp png_ptr, png_bytep data, png_size_t length)
    else
    {
       png_byte buf[4];
-      
+
       png_write_chunk_start(png_ptr, (png_bytep)png_fdAT, 4 + length);
-      
+
       png_save_uint_32(buf, png_ptr->next_seq_num);
       png_write_chunk_data(png_ptr, buf, 4);
-      
+
       png_write_chunk_data(png_ptr, data, length);
-      
+
       png_write_chunk_end(png_ptr);
-      
+
       png_ptr->next_seq_num++;
    }
 #endif
@@ -1748,43 +1768,43 @@ png_write_acTL(png_structp png_ptr,
 {
     PNG_acTL;
     png_byte data[16];
-    
+
     png_debug(1, "in png_write_acTL");
-    
+
     png_ptr->num_frames_to_write = num_frames;
-    
+
     if (png_ptr->apng_flags & PNG_FIRST_FRAME_HIDDEN)
         num_frames--;
-    
+
     png_save_uint_32(data, num_frames);
     png_save_uint_32(data + 4, num_plays);
-    
+
     png_write_chunk(png_ptr, (png_bytep)png_acTL, data, (png_size_t)8);
 }
 
 void /* PRIVATE */
-png_write_fcTL(png_structp png_ptr, png_uint_32 width, png_uint_32 height, 
+png_write_fcTL(png_structp png_ptr, png_uint_32 width, png_uint_32 height,
     png_uint_32 x_offset, png_uint_32 y_offset,
-    png_uint_16 delay_num, png_uint_16 delay_den, png_byte dispose_op, 
+    png_uint_16 delay_num, png_uint_16 delay_den, png_byte dispose_op,
     png_byte blend_op)
 {
     PNG_fcTL;
     png_byte data[26];
-    
+
     png_debug(1, "in png_write_fcTL");
-    
+
     if (png_ptr->num_frames_written == 0 && (x_offset != 0 || y_offset != 0))
         png_error(png_ptr, "x and/or y offset for the first frame aren't 0");
-    if (png_ptr->num_frames_written == 0 && 
-        (width != png_ptr->first_frame_width || 
+    if (png_ptr->num_frames_written == 0 &&
+        (width != png_ptr->first_frame_width ||
          height != png_ptr->first_frame_height))
         png_error(png_ptr, "width and/or height in the first frame's fcTL "
                            "don't match the ones in IHDR");
-    
+
     /* more error checking */
-    png_ensure_fcTL_is_valid(png_ptr, width, height, x_offset, y_offset, 
+    png_ensure_fcTL_is_valid(png_ptr, width, height, x_offset, y_offset,
                              delay_num, delay_den, dispose_op, blend_op);
-    
+
     png_save_uint_32(data, png_ptr->next_seq_num);
     png_save_uint_32(data + 4, width);
     png_save_uint_32(data + 8, height);
@@ -1794,9 +1814,9 @@ png_write_fcTL(png_structp png_ptr, png_uint_32 width, png_uint_32 height,
     png_save_uint_16(data + 22, delay_den);
     data[24] = dispose_op;
     data[25] = blend_op;
-    
+
     png_write_chunk(png_ptr, (png_bytep)png_fcTL, data, (png_size_t)26);
-    
+
     png_ptr->next_seq_num++;
 }
 #endif /* PNG_WRITE_APNG_SUPPORTED */
@@ -2883,24 +2903,24 @@ png_write_reset(png_structp png_ptr)
 }
 
 void /* PRIVATE */
-png_write_reinit(png_structp png_ptr, png_infop info_ptr, 
+png_write_reinit(png_structp png_ptr, png_infop info_ptr,
                  png_uint_32 width, png_uint_32 height)
 {
-    if (png_ptr->num_frames_written == 0 && 
-        (width != png_ptr->first_frame_width || 
+    if (png_ptr->num_frames_written == 0 &&
+        (width != png_ptr->first_frame_width ||
          height != png_ptr->first_frame_height))
         png_error(png_ptr, "width and/or height in the first frame's fcTL "
                            "don't match the ones in IHDR");
-    if (width > png_ptr->first_frame_width || 
+    if (width > png_ptr->first_frame_width ||
         height > png_ptr->first_frame_height)
         png_error(png_ptr, "width and/or height for a frame greater than"
                            "the ones in IHDR");
-    
-    png_set_IHDR(png_ptr, info_ptr, width, height, 
-                 info_ptr->bit_depth, info_ptr->color_type, 
+
+    png_set_IHDR(png_ptr, info_ptr, width, height,
+                 info_ptr->bit_depth, info_ptr->color_type,
                  info_ptr->interlace_type, info_ptr->compression_type,
                  info_ptr->filter_type);
-   
+
     png_ptr->width = width;
     png_ptr->height = height;
     png_ptr->rowbytes = PNG_ROWBYTES(png_ptr->pixel_depth, width);
