@@ -182,6 +182,7 @@ nsIOService::nsIOService()
     , mShutdown(PR_FALSE)
     , mChannelEventSinks(NS_CHANNEL_EVENT_SINK_CATEGORY)
     , mContentSniffers(NS_CONTENT_SNIFFER_CATEGORY)
+    , mAutoDialEnabled(PR_FALSE)
 {
 }
 
@@ -304,6 +305,7 @@ nsIOService::InitializeSocketTransportService()
     if (mSocketTransportService) {
         rv = mSocketTransportService->Init();
         NS_ASSERTION(NS_SUCCEEDED(rv), "socket transport service init failed");
+        mSocketTransportService->SetAutodialEnabled(mAutoDialEnabled);
     }
 
     return rv;
@@ -593,6 +595,15 @@ nsIOService::NewFileURI(nsIFile *file, nsIURI **result)
 
 NS_IMETHODIMP
 nsIOService::NewChannelFromURI(nsIURI *aURI, nsIChannel **result)
+{
+    return NewChannelFromURIWithProxyFlags(aURI, nsnull, 0, result);
+}
+
+NS_IMETHODIMP
+nsIOService::NewChannelFromURIWithProxyFlags(nsIURI *aURI,
+                                             nsIURI *aProxyURI,
+                                             PRUint32 proxyFlags,
+                                             nsIChannel **result)
 {
     nsresult rv;
     NS_ENSURE_ARG_POINTER(aURI);
@@ -623,7 +634,8 @@ nsIOService::NewChannelFromURI(nsIURI *aURI, nsIChannel **result)
                 NS_WARNING("failed to get protocol proxy service");
         }
         if (mProxyService) {
-            rv = mProxyService->Resolve(aURI, 0, getter_AddRefs(pi));
+            rv = mProxyService->Resolve(aProxyURI ? aProxyURI : aURI,
+                                        proxyFlags, getter_AddRefs(pi));
             if (NS_FAILED(rv))
                 pi = nsnull;
         }
@@ -850,6 +862,7 @@ nsIOService::PrefsChanged(nsIPrefBranch *prefs, const char *pref)
         PRBool enableAutodial = PR_FALSE;
         nsresult rv = prefs->GetBoolPref(AUTODIAL_PREF, &enableAutodial);
         // If pref not found, default to disabled.
+        mAutoDialEnabled = enableAutodial;
         if (NS_SUCCEEDED(rv)) {
             if (mSocketTransportService)
                 mSocketTransportService->SetAutodialEnabled(enableAutodial);
