@@ -43,7 +43,6 @@
 #include "nsIServiceManager.h"
 #include "nsXPIDLString.h"
 #include "nsIProxyAutoConfig.h"
-#include "nsAutoLock.h"
 #include "nsIIOService.h"
 #include "nsIObserverService.h"
 #include "nsIProtocolHandler.h"
@@ -56,6 +55,7 @@
 #include "nsThreadUtils.h"
 #include "nsString.h"
 #include "nsNetUtil.h"
+#include "nsNetCID.h"
 #include "nsCRT.h"
 #include "prnetdb.h"
 #include "nsPACMan.h"
@@ -94,9 +94,11 @@ public:
     NS_DECL_ISUPPORTS
 
     nsAsyncResolveRequest(nsProtocolProxyService *pps, nsIURI *uri,
+                          PRUint32 aResolveFlags,
                           nsIProtocolProxyCallback *callback)
         : mStatus(NS_OK)
         , mDispatched(PR_FALSE)
+        , mResolveFlags(0)
         , mPPS(pps)
         , mURI(uri)
         , mCallback(callback)
@@ -172,7 +174,8 @@ private:
     {
         // Generate proxy info from the PAC string if appropriate
         if (NS_SUCCEEDED(mStatus) && !mProxyInfo && !mPACString.IsEmpty())
-            mPPS->ProcessPACString(mPACString, getter_AddRefs(mProxyInfo));
+            mPPS->ProcessPACString(mPACString, mResolveFlags,
+                                   getter_AddRefs(mProxyInfo));
 
         // Now apply proxy filters
         if (NS_SUCCEEDED(mStatus)) {
@@ -193,6 +196,7 @@ private:
     nsresult  mStatus;
     nsCString mPACString;
     PRBool    mDispatched;
+    PRUint32  mResolveFlags;
 
     nsRefPtr<nsProtocolProxyService>   mPPS;
     nsCOMPtr<nsIURI>                   mURI;
@@ -286,8 +290,13 @@ proxy_GetBoolPref(nsIPrefBranch *aPrefBranch,
 
 //----------------------------------------------------------------------------
 
+static const PRInt32 PROXYCONFIG_DIRECT4X = 3;
+static const PRInt32 PROXYCONFIG_COUNT = 6;
+
 NS_IMPL_ADDREF(nsProtocolProxyService)
 NS_IMPL_RELEASE(nsProtocolProxyService)
+NS_IMPL_CLASSINFO(nsProtocolProxyService, NULL, nsIClassInfo::SINGLETON,
+                  NS_PROTOCOLPROXYSERVICE_CID)
 NS_IMPL_QUERY_INTERFACE3_CI(nsProtocolProxyService,
                             nsIProtocolProxyService,
                             nsIProtocolProxyService2,
@@ -298,10 +307,9 @@ NS_IMPL_CI_INTERFACE_GETTER2(nsProtocolProxyService,
 
 nsProtocolProxyService::nsProtocolProxyService()
     : mFilters(nsnull)
-    , mProxyConfig(eProxyConfig_Direct)
+    , mProxyConfig(PROXYCONFIG_DIRECT)
     , mHTTPProxyPort(-1)
     , mFTPProxyPort(-1)
-    , mGopherProxyPort(-1)
     , mHTTPSProxyPort(-1)
     , mSOCKSProxyPort(-1)
     , mSOCKSProxyVersion(4)
@@ -315,7 +323,7 @@ nsProtocolProxyService::nsProtocolProxyService()
 nsProtocolProxyService::~nsProtocolProxyService()
 {
     // These should have been cleaned up in our Observe method.
-    NS_ASSERTION(mHostFiltersArray.Count() == 0 && mFilters == nsnull &&
+    NS_ASSERTION(mHostFiltersArray.Length() == 0 && mFilters == nsnull &&
                  mPACMan == nsnull, "what happened to xpcom-shutdown?");
 }
 
@@ -338,8 +346,7 @@ nsProtocolProxyService::Init()
     }
 
     // register for shutdown notification so we can clean ourselves up properly.
-    nsCOMPtr<nsIObserverService> obs =
-            do_GetService("@mozilla.org/observer-service;1");
+    nsCOMPtr<nsIObserverService> obs = mozilla::services::GetObserverService();
     if (obs)
         obs->AddObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID, PR_FALSE);
 
@@ -353,8 +360,7 @@ nsProtocolProxyService::Observe(nsISupports     *aSubject,
 {
     if (strcmp(aTopic, NS_XPCOM_SHUTDOWN_OBSERVER_ID) == 0) {
         // cleanup
-        if (mHostFiltersArray.Count() > 0) {
-            mHostFiltersArray.EnumerateForwards(CleanupFilterArray, nsnull);
+        if (mHostFiltersArray.Length() > 0) {
             mHostFiltersArray.Clear();
         }
         if (mFilters) {
@@ -389,24 +395,26 @@ nsProtocolProxyService::PrefsChanged(nsIPrefBranch *prefBranch,
         rv = prefBranch->GetIntPref(PROXY_PREF("type"), &type);
         if (NS_SUCCEEDED(rv)) {
             // bug 115720 - for ns4.x backwards compatability
-            if (type == eProxyConfig_Direct4x) {
-                type = eProxyConfig_Direct;
+            if (type == PROXYCONFIG_DIRECT4X) {
+                type = PROXYCONFIG_DIRECT;
                 // Reset the type so that the dialog looks correct, and we
                 // don't have to handle this case everywhere else
                 // I'm paranoid about a loop of some sort - only do this
                 // if we're enumerating all prefs, and ignore any error
                 if (!pref)
                     prefBranch->SetIntPref(PROXY_PREF("type"), type);
-            } else if (type >= eProxyConfig_Last) {
+            } else if (type >= PROXYCONFIG_COUNT) {
                 LOG(("unknown proxy type: %lu; assuming direct\n", type));
-                type = eProxyConfig_Direct;
+                type = PROXYCONFIG_DIRECT;
             }
-            mProxyConfig = static_cast<ProxyConfig>(type);
+            mProxyConfig = type;
             reloadPAC = PR_TRUE;
         }
 
-        if (mProxyConfig == eProxyConfig_System) {
+        if (mProxyConfig == PROXYCONFIG_SYSTEM) {
             mSystemProxySettings = do_GetService(NS_SYSTEMPROXYSETTINGS_CONTRACTID);
+            if (!mSystemProxySettings)
+                mProxyConfig = PROXYCONFIG_DIRECT;
         } else {
             mSystemProxySettings = nsnull;
         }
@@ -430,12 +438,6 @@ nsProtocolProxyService::PrefsChanged(nsIPrefBranch *prefBranch,
     if (!pref || !strcmp(pref, PROXY_PREF("ftp_port")))
         proxy_GetIntPref(prefBranch, PROXY_PREF("ftp_port"), mFTPProxyPort);
 
-    if (!pref || !strcmp(pref, PROXY_PREF("gopher")))
-        proxy_GetStringPref(prefBranch, PROXY_PREF("gopher"), mGopherProxyHost);
-
-    if (!pref || !strcmp(pref, PROXY_PREF("gopher_port")))
-        proxy_GetIntPref(prefBranch, PROXY_PREF("gopher_port"), mGopherProxyPort);
-
     if (!pref || !strcmp(pref, PROXY_PREF("socks")))
         proxy_GetStringPref(prefBranch, PROXY_PREF("socks"), mSOCKSProxyHost);
     
@@ -469,8 +471,8 @@ nsProtocolProxyService::PrefsChanged(nsIPrefBranch *prefBranch,
 
     // We're done if not using something that could give us a PAC URL
     // (PAC, WPAD or System)
-    if (mProxyConfig != eProxyConfig_PAC && mProxyConfig != eProxyConfig_WPAD &&
-        mProxyConfig != eProxyConfig_System)
+    if (mProxyConfig != PROXYCONFIG_PAC && mProxyConfig != PROXYCONFIG_WPAD &&
+        mProxyConfig != PROXYCONFIG_SYSTEM)
         return;
 
     // OK, we need to reload the PAC file if:
@@ -482,10 +484,10 @@ nsProtocolProxyService::PrefsChanged(nsIPrefBranch *prefBranch,
 
     if (reloadPAC) {
         tempString.Truncate();
-        if (mProxyConfig == eProxyConfig_PAC) {
+        if (mProxyConfig == PROXYCONFIG_PAC) {
             prefBranch->GetCharPref(PROXY_PREF("autoconfig_url"),
                                     getter_Copies(tempString));
-        } else if (mProxyConfig == eProxyConfig_WPAD) {
+        } else if (mProxyConfig == PROXYCONFIG_WPAD) {
             // We diverge from the WPAD spec here in that we don't walk the
             // hosts's FQDN, stripping components until we hit a TLD.  Doing so
             // is dangerous in the face of an incomplete list of TLDs, and TLDs
@@ -505,7 +507,7 @@ nsProtocolProxyService::PrefsChanged(nsIPrefBranch *prefBranch,
 PRBool
 nsProtocolProxyService::CanUseProxy(nsIURI *aURI, PRInt32 defaultPort) 
 {
-    if (mHostFiltersArray.Count() == 0)
+    if (mHostFiltersArray.Length() == 0)
         return PR_TRUE;
 
     PRInt32 port;
@@ -542,8 +544,8 @@ nsProtocolProxyService::CanUseProxy(nsIURI *aURI, PRInt32 defaultPort)
     }
     
     PRInt32 index = -1;
-    while (++index < mHostFiltersArray.Count()) {
-        HostInfo *hinfo = (HostInfo *) mHostFiltersArray[index];
+    while (++index < PRInt32(mHostFiltersArray.Length())) {
+        HostInfo *hinfo = mHostFiltersArray[index];
 
         if (is_ipaddr != hinfo->is_ipaddr)
             continue;
@@ -586,7 +588,9 @@ static const char kProxyType_DIRECT[]  = "direct";
 static const char kProxyType_UNKNOWN[] = "unknown";
 
 const char *
-nsProtocolProxyService::ExtractProxyInfo(const char *start, nsProxyInfo **result)
+nsProtocolProxyService::ExtractProxyInfo(const char *start,
+                                         PRUint32 aResolveFlags,
+                                         nsProxyInfo **result)
 {
     *result = nsnull;
     PRUint32 flags = 0;
@@ -653,6 +657,7 @@ nsProtocolProxyService::ExtractProxyInfo(const char *start, nsProxyInfo **result
         if (pi) {
             pi->mType = type;
             pi->mFlags = flags;
+            pi->mResolveFlags = aResolveFlags;
             pi->mTimeout = mFailedProxyTimeout;
             // YES, it is ok to specify a null proxy host.
             if (host) {
@@ -722,7 +727,7 @@ nsProtocolProxyService::DisableProxy(nsProxyInfo *pi)
     dsec += pi->mTimeout;
 
     // NOTE: The classic codebase would increase the timeout value
-    //       incrementally each time a subsequent failure occured.
+    //       incrementally each time a subsequent failure occurred.
     //       We could do the same, but it would require that we not
     //       remove proxy entries in IsProxyDisabled or otherwise
     //       change the way we are recording disabled proxies.
@@ -782,6 +787,7 @@ nsProtocolProxyService::ConfigureFromPAC(const nsCString &spec,
 
 void
 nsProtocolProxyService::ProcessPACString(const nsCString &pacString,
+                                         PRUint32 aResolveFlags,
                                          nsIProxyInfo **result)
 {
     if (pacString.IsEmpty()) {
@@ -793,7 +799,7 @@ nsProtocolProxyService::ProcessPACString(const nsCString &pacString,
 
     nsProxyInfo *pi = nsnull, *first = nsnull, *last = nsnull;
     while (*proxies) {
-        proxies = ExtractProxyInfo(proxies, &pi);
+        proxies = ExtractProxyInfo(proxies, aResolveFlags, &pi);
         if (pi) {
             if (last) {
                 NS_ASSERTION(last->mNext == nsnull, "leaking nsProxyInfo");
@@ -821,9 +827,9 @@ nsProtocolProxyService::ReloadPAC()
         return NS_OK;
 
     nsXPIDLCString pacSpec;
-    if (type == eProxyConfig_PAC)
+    if (type == PROXYCONFIG_PAC)
         prefs->GetCharPref(PROXY_PREF("autoconfig_url"), getter_Copies(pacSpec));
-    else if (type == eProxyConfig_WPAD)
+    else if (type == PROXYCONFIG_WPAD)
         pacSpec.AssignLiteral(WPAD_URL);
 
     if (!pacSpec.IsEmpty())
@@ -842,7 +848,7 @@ nsProtocolProxyService::Resolve(nsIURI *uri, PRUint32 flags,
         return rv;
 
     PRBool usePAC;
-    rv = Resolve_Internal(uri, info, &usePAC, result);
+    rv = Resolve_Internal(uri, info, flags, &usePAC, result);
     if (NS_FAILED(rv))
         return rv;
 
@@ -857,12 +863,12 @@ nsProtocolProxyService::Resolve(nsIURI *uri, PRUint32 flags,
         nsCString pacString;
         rv = mPACMan->GetProxyForURI(uri, pacString);
         if (NS_SUCCEEDED(rv))
-            ProcessPACString(pacString, result);
+            ProcessPACString(pacString, flags, result);
         else if (rv == NS_ERROR_IN_PROGRESS) {
             // Construct a special UNKNOWN proxy entry that informs the caller
             // that the proxy info is yet to be determined.
             rv = NewProxyInfo_Internal(kProxyType_UNKNOWN, EmptyCString(), -1,
-                                       0, 0, nsnull, result);
+                                       0, 0, nsnull, flags, result);
             if (NS_FAILED(rv))
                 return rv;
         }
@@ -880,7 +886,7 @@ nsProtocolProxyService::AsyncResolve(nsIURI *uri, PRUint32 flags,
                                      nsICancelable **result)
 {
     nsRefPtr<nsAsyncResolveRequest> ctx =
-            new nsAsyncResolveRequest(this, uri, callback);
+        new nsAsyncResolveRequest(this, uri, flags, callback);
     if (!ctx)
         return NS_ERROR_OUT_OF_MEMORY;
 
@@ -891,7 +897,7 @@ nsProtocolProxyService::AsyncResolve(nsIURI *uri, PRUint32 flags,
 
     PRBool usePAC;
     nsCOMPtr<nsIProxyInfo> pi;
-    rv = Resolve_Internal(uri, info, &usePAC, getter_AddRefs(pi));
+    rv = Resolve_Internal(uri, info, flags, &usePAC, getter_AddRefs(pi));
     if (NS_FAILED(rv))
         return rv;
 
@@ -942,7 +948,7 @@ nsProtocolProxyService::NewProxyInfo(const nsACString &aType,
         aPort = -1;
 
     return NewProxyInfo_Internal(type, aHost, aPort, aFlags, aFailoverTimeout,
-                                 aFailoverProxy, aResult);
+                                 aFailoverProxy, 0, aResult);
 }
 
 NS_IMETHODIMP
@@ -953,8 +959,8 @@ nsProtocolProxyService::GetFailoverForProxy(nsIProxyInfo  *aProxy,
 {
     // We only support failover when a PAC file is configured, either
     // directly or via system settings
-    if (mProxyConfig != eProxyConfig_PAC && mProxyConfig != eProxyConfig_WPAD &&
-        mProxyConfig != eProxyConfig_System)
+    if (mProxyConfig != PROXYCONFIG_PAC && mProxyConfig != PROXYCONFIG_WPAD &&
+        mProxyConfig != PROXYCONFIG_SYSTEM)
         return NS_ERROR_NOT_AVAILABLE;
 
     // Verify that |aProxy| is one of our nsProxyInfo objects.
@@ -1041,21 +1047,18 @@ nsProtocolProxyService::UnregisterFilter(nsIProtocolProxyFilter *filter)
     return NS_OK;
 }
 
-PRBool
-nsProtocolProxyService::CleanupFilterArray(void *aElement, void *aData) 
+NS_IMETHODIMP
+nsProtocolProxyService::GetProxyConfigType(PRUint32* aProxyConfigType)
 {
-    if (aElement)
-        delete (HostInfo *) aElement;
-
-    return PR_TRUE;
+  *aProxyConfigType = mProxyConfig;
+  return NS_OK;
 }
 
 void
 nsProtocolProxyService::LoadHostFilters(const char *filters)
 {
     // check to see the owners flag? /!?/ TODO
-    if (mHostFiltersArray.Count() > 0) {
-        mHostFiltersArray.EnumerateForwards(CleanupFilterArray, nsnull);
+    if (mHostFiltersArray.Length() > 0) {
         mHostFiltersArray.Clear();
     }
 
@@ -1076,15 +1079,13 @@ nsProtocolProxyService::LoadHostFilters(const char *filters)
         const char *portLocation = 0; 
         const char *maskLocation = 0;
 
-        //
-        // XXX this needs to be fixed to support IPv6 address literals,
-        // which in this context will need to be []-escaped.
-        //
         while (*endhost && (*endhost != ',' && !IS_ASCII_SPACE(*endhost))) {
             if (*endhost == ':')
                 portLocation = endhost;
             else if (*endhost == '/')
                 maskLocation = endhost;
+            else if (*endhost == ']') // IPv6 address literals
+                portLocation = 0;
             endhost++;
         }
 
@@ -1151,7 +1152,7 @@ nsProtocolProxyService::LoadHostFilters(const char *filters)
 
 //#define DEBUG_DUMP_FILTERS
 #ifdef DEBUG_DUMP_FILTERS
-        printf("loaded filter[%u]:\n", mHostFiltersArray.Count());
+        printf("loaded filter[%u]:\n", mHostFiltersArray.Length());
         printf("  is_ipaddr = %u\n", hinfo->is_ipaddr);
         printf("  port = %u\n", hinfo->port);
         if (hinfo->is_ipaddr) {
@@ -1175,8 +1176,7 @@ nsProtocolProxyService::LoadHostFilters(const char *filters)
         mHostFiltersArray.AppendElement(hinfo);
         hinfo = nsnull;
 loser:
-        if (hinfo)
-            delete hinfo;
+        delete hinfo;
     }
 }
 
@@ -1213,6 +1213,7 @@ nsProtocolProxyService::NewProxyInfo_Internal(const char *aType,
                                               PRUint32 aFlags,
                                               PRUint32 aFailoverTimeout,
                                               nsIProxyInfo *aFailoverProxy,
+                                              PRUint32 aResolveFlags,
                                               nsIProxyInfo **aResult)
 {
     nsCOMPtr<nsProxyInfo> failover;
@@ -1229,6 +1230,7 @@ nsProtocolProxyService::NewProxyInfo_Internal(const char *aType,
     proxyInfo->mHost = aHost;
     proxyInfo->mPort = aPort;
     proxyInfo->mFlags = aFlags;
+    proxyInfo->mResolveFlags = aResolveFlags;
     proxyInfo->mTimeout = aFailoverTimeout == PR_UINT32_MAX
         ? mFailedProxyTimeout : aFailoverTimeout;
     failover.swap(proxyInfo->mNext);
@@ -1240,6 +1242,7 @@ nsProtocolProxyService::NewProxyInfo_Internal(const char *aType,
 nsresult
 nsProtocolProxyService::Resolve_Internal(nsIURI *uri,
                                          const nsProtocolInfo &info,
+                                         PRUint32 flags,
                                          PRBool *usePAC,
                                          nsIProxyInfo **result)
 {
@@ -1253,35 +1256,41 @@ nsProtocolProxyService::Resolve_Internal(nsIURI *uri,
 
     if (mSystemProxySettings) {
         nsCAutoString PACURI;
-        if (NS_SUCCEEDED(mSystemProxySettings->GetPACURI(PACURI)) &&
-            !PACURI.IsEmpty()) {
-            // Switch to new PAC file if that setting has changed. If the setting
-            // hasn't changed, ConfigureFromPAC will exit early.
-            nsresult rv = ConfigureFromPAC(PACURI, PR_FALSE);
-            if (NS_FAILED(rv))
-                return rv;
-        } else {
+        if (NS_FAILED(mSystemProxySettings->GetPACURI(PACURI)) ||
+            PACURI.IsEmpty()) {
             nsCAutoString proxy;
             nsresult rv = mSystemProxySettings->GetProxyForURI(uri, proxy);
             if (NS_SUCCEEDED(rv)) {
-                ProcessPACString(proxy, result);
+                ProcessPACString(proxy, flags, result);
                 return NS_OK;
             }
             // no proxy, stop search
             return NS_OK;
         }
+
+        // See bug #586908.
+        // Avoid endless loop if |uri| is the current PAC-URI. Returning OK
+        // here means that we will not use a proxy for this connection.
+        if (mPACMan && mPACMan->IsPACURI(uri))
+            return NS_OK;
+
+        // Switch to new PAC file if that setting has changed. If the setting
+        // hasn't changed, ConfigureFromPAC will exit early.
+        nsresult rv = ConfigureFromPAC(PACURI, PR_FALSE);
+        if (NS_FAILED(rv))
+            return rv;
     }
 
     // if proxies are enabled and this host:port combo is supposed to use a
     // proxy, check for a proxy.
-    if (mProxyConfig == eProxyConfig_Direct ||
-            (mProxyConfig == eProxyConfig_Manual &&
-             !CanUseProxy(uri, info.defaultPort)))
+    if (mProxyConfig == PROXYCONFIG_DIRECT ||
+        (mProxyConfig == PROXYCONFIG_MANUAL &&
+         !CanUseProxy(uri, info.defaultPort)))
         return NS_OK;
 
     // Proxy auto config magic...
-    if (mProxyConfig == eProxyConfig_PAC || mProxyConfig == eProxyConfig_WPAD ||
-        mProxyConfig == eProxyConfig_System) {
+    if (mProxyConfig == PROXYCONFIG_PAC || mProxyConfig == PROXYCONFIG_WPAD ||
+        mProxyConfig == PROXYCONFIG_SYSTEM) {
         // Do not query PAC now.
         *usePAC = PR_TRUE;
         return NS_OK;
@@ -1294,30 +1303,44 @@ nsProtocolProxyService::Resolve_Internal(nsIURI *uri,
 
     PRUint32 proxyFlags = 0;
 
-    if (!mHTTPProxyHost.IsEmpty() && mHTTPProxyPort > 0 &&
-        info.scheme.EqualsLiteral("http")) {
+    if ((flags & RESOLVE_PREFER_SOCKS_PROXY) &&
+        !mSOCKSProxyHost.IsEmpty() && mSOCKSProxyPort > 0) {
+      host = &mSOCKSProxyHost;
+      if (mSOCKSProxyVersion == 4) 
+          type = kProxyType_SOCKS4;
+      else
+          type = kProxyType_SOCKS;
+      port = mSOCKSProxyPort;
+      if (mSOCKSProxyRemoteDNS)
+          proxyFlags |= nsIProxyInfo::TRANSPARENT_PROXY_RESOLVES_HOST;
+    }
+    else if ((flags & RESOLVE_PREFER_HTTPS_PROXY) &&
+             !mHTTPSProxyHost.IsEmpty() && mHTTPSProxyPort > 0) {
+        host = &mHTTPSProxyHost;
+        type = kProxyType_HTTP;
+        port = mHTTPSProxyPort;
+    }
+    else if (!mHTTPProxyHost.IsEmpty() && mHTTPProxyPort > 0 &&
+             ((flags & RESOLVE_IGNORE_URI_SCHEME) ||
+              info.scheme.EqualsLiteral("http"))) {
         host = &mHTTPProxyHost;
         type = kProxyType_HTTP;
         port = mHTTPProxyPort;
     }
     else if (!mHTTPSProxyHost.IsEmpty() && mHTTPSProxyPort > 0 &&
+             !(flags & RESOLVE_IGNORE_URI_SCHEME) &&
              info.scheme.EqualsLiteral("https")) {
         host = &mHTTPSProxyHost;
         type = kProxyType_HTTP;
         port = mHTTPSProxyPort;
     }
     else if (!mFTPProxyHost.IsEmpty() && mFTPProxyPort > 0 &&
+             !(flags & RESOLVE_IGNORE_URI_SCHEME) &&
              info.scheme.EqualsLiteral("ftp")) {
         host = &mFTPProxyHost;
         type = kProxyType_HTTP;
         port = mFTPProxyPort;
     }
-    else if (!mGopherProxyHost.IsEmpty() && mGopherProxyPort > 0 &&
-             info.scheme.EqualsLiteral("gopher")) {
-        host = &mGopherProxyHost;
-        type = kProxyType_HTTP;
-        port = mGopherProxyPort;
-    }
     else if (!mSOCKSProxyHost.IsEmpty() && mSOCKSProxyPort > 0) {
         host = &mSOCKSProxyHost;
         if (mSOCKSProxyVersion == 4) 
@@ -1331,7 +1354,8 @@ nsProtocolProxyService::Resolve_Internal(nsIURI *uri,
 
     if (type) {
         nsresult rv = NewProxyInfo_Internal(type, *host, port, proxyFlags,
-                                            PR_UINT32_MAX, nsnull, result);
+                                            PR_UINT32_MAX, nsnull, flags,
+                                            result);
         if (NS_FAILED(rv))
             return rv;
     }
